"""
##############################################################################
#
# Copyright (c) 2003-2005 struktur AG and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# $Id: SquidTool.py,v 1.12 2005/03/01 11:41:07 longsleep Exp $ (Author: $Author: longsleep $)
"""

# make sockets non blocking
import timeoutsocket
timeoutsocket.setDefaultSocketTimeout(5)

import os, re, httplib, urlparse, urllib, sys
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, package_home
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.Expression import Expression
from Products.CMFCore.CMFCorePermissions  import ManagePortal, ModifyPortalContent, View
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Acquisition import aq_base
from Permissions import *

URL_REWRITE_MAP = {}

# urlparse.urljoin does relative urls.
# we want simple path appending. 
def joinUrlPath(url, path2):
    (scheme, host, path, params, query, frag) = urlparse.urlparse(url)
    path = path.rstrip('/') + '/' +  path2.lstrip('/')
    return urlparse.urlunparse((scheme, host, path, None, None, None))
        
class SquidTool(UniqueObject, SimpleItem):
    """ Tool to send PURGE requests to caching proxy such as Squid or EEP.  """

    id        = 'portal_squid'
    meta_type = 'CMF Squid Tool'

    security = ClassSecurityInfo()

    manage_options=(
        ({ 'label'   : 'Proxy Urls',
           'action'  : 'manage_configForm',
           },
         ) + SimpleItem.manage_options
        )

    manage_configForm = PageTemplateFile('www/config', globals())

    def __init__(self):
        self.id = 'portal_squid'
        self.squid_urls = []
        self.cachetype= 'eep'

    security.declareProtected(ManagePortal, 'setCacheType')
    def setCacheType(self, c, REQUEST=None):
        """ Sets the cachetypes """
        assert c in ["eep", "squid"]
        self.cachetype = c
        if REQUEST:
            REQUEST.RESPONSE.redirect(REQUEST['HTTP_REFERER'])

    security.declarePublic('getCacheType')
    def getCacheType(self):
        """ Returns the cachetype, eep or squid """
        return self.cachetype

    security.declareProtected(ManagePortal, 'manage_setSquidSettings')
    def manage_setSquidSettings(self, urls, REQUEST=None):
        ''' stores the tool settings '''

        urls = urls.replace("\r\n", "\n")
        urls = urls.split("\n")
        urls = map(lambda x: x.strip(), urls)
        urls = filter(lambda x: x, urls)
        self.squid_urls = urls 
            
        if REQUEST:
            url = REQUEST['HTTP_REFERER'].split('?')[0]
            url += '?portal_status_message=Settings updated'
            REQUEST.RESPONSE.redirect(url)


    security.declarePrivate('pruneUrl')
    def pruneUrl(self, ob_url=None, purge_type="PURGE", REQUEST=None):
        # ob_url is a relative to portal url

        results = []
        if ob_url:
            ob_url = self.rewriteUrl(ob_url)
            ob_url = urllib.quote(ob_url)

        for url in self.squid_urls:
            # XXX: probably put into seperate thread
            #      and setup a Queue

            if ob_url:
                url = joinUrlPath(url, ob_url)
            
            (scheme, host, path, params, query, fragment) = urlparse.urlparse(url)

            try:
                conn = httplib.HTTPConnection(host)
                 
                if self.cachetype == 'eep':
                    conn.request(purge_type, path)
                elif self.cachetype == 'squid' or self.cachetype is null:
                    if purge_type != "PURGE":
                        raise AttributeError, "Squid only understands PURGE"
                    # XXX hack to use HTTP/1.0 and the timeoutsocket
                    # NOTE: only HTTP/1.0 and the full url can make PURGE work
                    # You can see code from proxy's client.c
                    conn._http_vsn = 10
                    conn.conn_http_vsn_str = 'HTTP/1.0'
                    conn.putrequest(purge_type, url) 
                    conn.endheaders()
                    conn.sock.set_timeout(2)
                else:
                    raise "Don't know how to PURGE an unknown cache type: '%s'" + self.cachetype
                resp = conn.getresponse()
                status = resp.status
                xcache = resp.getheader('x-cache', '')
                xerror = resp.getheader('x-%s-error' % self.cachetype, '')
            except:
                err, msg, tb = sys.exc_info()
                try:
                    xerror = "%s %s" % (err[0], err[1])
                except TypeError:
                    xerror = "%s" % err
                import traceback
                xerror = traceback.format_exception(err, msg, tb)
                status = 000 
                xcache = ''

            results.append((status, xcache, xerror))

            # NOTE: if the purge was successfull status will be 200 (OK)
            #       if the object was not in cache status is 404 (NOT FOUND)
            #       if you are not allowed to PURGE status is 403
            #       see README.txt for details how to setup proxy to allow PURGE

            if REQUEST: REQUEST.RESPONSE.write('%s\t%s\t%s\n' % (status, url, xerror))

        return results

    security.declarePublic('rewriteUrl')
    def rewriteUrl(self, url):
        for prefix, new in URL_REWRITE_MAP.items():
            if url.startswith(prefix):
                url = url.replace(prefix, new)
                break
        return url

    security.declareProtected(PURGE_URL, 'pruneObject')
    def pruneObject(self, ob, REQUEST=None):
        # prune this object
        portal_url = getToolByName(self, 'portal_url')
        url = portal_url.getRelativeUrl(ob)
        return self.pruneUrl(url, REQUEST=REQUEST)        

    security.declareProtected(PURGE_URL, 'manage_pruneUrl')
    def manage_pruneUrl(self, url, REQUEST=None):
        """ give a url which shall be pruned """
        request = REQUEST 
        return self.pruneUrl(url, REQUEST=REQUEST)

    security.declareProtected(ManagePortal, 'manage_pruneAll')
    def manage_pruneAll(self, REQUEST=None):
        """ prune all objects in catalog """
        #import pdb; pdb.set_trace()
        if self.getCacheType() == 'eep':
            self.pruneUrl(purge_type="PURGEALL", REQUEST=REQUEST)
        else:
            portal_catalog = getToolByName(self, 'portal_catalog')
            brains = portal_catalog()
            for brain in brains:
                url = brain.getURL()
                self.pruneUrl(url, REQUEST=REQUEST)

        #if REQUEST:
        #   url = REQUEST['HTTP_REFERER'].split('?')[0]
        #   url += '?portal_status_message=Cache+cleared'
        #   REQUEST.RESPONSE.redirect(url)

        #return "finished"
 
InitializeClass(SquidTool)
