# This product is licensed under the GNU Lesser General Public License:
#
# ZipCodeTool  |  Copyright(C), 2004, Enfold Systems, LLC
#
# Enfold Systems, LLC
# 4500 Montrose Blvd., Suite D
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# For a full copy of the license, please see the included 'LGPL.txt' file.

from BTrees.IOBTree import IOBTree
from Products.CMFCore.utils import UniqueObject
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from zipdata import zip2state
from utils import calculateDistance
from config import distance_ranges
distance_ranges.sort()
range_count = len(distance_ranges)

class ZipCode:

    def __init__(self, zip, zranges):
        self.zip = int(zip)
        assert len(zranges) == range_count
        ranges = {}
        for i in range(range_count):
            ranges[distance_ranges[i]] = zranges[i]
        self.ranges = ranges
        

class ZipCodeTool(UniqueObject, SimpleItem):
    """
        ZipCode Tool
    """

    id = 'portal_zipcode'
    meta_type = 'ZipCode Tool'

    security = ClassSecurityInfo()
    _storage = None

    def __init__(self):
        self._storage = IOBTree()
    
    security.declarePrivate('addZip')
    def addZip(self, zipobj):
        """
            Add a zipcode to the database
        """
        key = zipobj.zip
        self._storage[int(key)] = zipobj

    security.declarePublic('getZip')
    def getZip(self, zip, default=None):
        """
            Get a zipcode object from the database
        """
        return self._storage.get(int(zip), default)

    security.declarePublic('getWithinRange')
    def getWithinRange(self, zip, zrange, default=()):
        """
            Returns a tuple of zipcodes within zrange miles of zip
            zrange must be in distance_ranges
        """
        zrange = int(zrange)
        assert zrange in distance_ranges
        zip = self.getZip(zip)
        if zip is None:
            return default
        zips = ()
        count = 0
        idx = distance_ranges.index(zrange)
        for r in distance_ranges:
            zips += tuple(zip.ranges[r])
            if count == idx:
                break
            count += 1
        return zips
        


    security.declarePublic('getState')
    def getState(self, zipcode):
        state = zip2state(zipcode)
        return state

    security.declarePublic('calculateDistance')
    def calculateDistance(self, zip1, zip2):
        return calculateDistance(zip1, zip2)

InitializeClass(ZipCodeTool)
