# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: adapter.py 758 2005-06-28 23:02:39Z sidnei $
"""

__metaclass__ = type

from zope.interface import implements

from zope.app import zapi
from zope.app.container.interfaces import IReadContainer
from zope.app.container.sample import SampleContainer
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent
from zope.app.servicenames import Authentication
from zope.app.security.settings import Allow, Deny, Unset
from zope.app.container.contained import Contained

from zope.app.securitypolicy.interfaces import IRole
from zope.app.securitypolicy.interfaces import IPrincipalRoleMap
from zope.security.proxy import removeSecurityProxy

from sqlobject import *

from sip.browser import api
from sip.interfaces.adapter import IPrincipalRolesHelper
from sip.descriptors import SecurityDescriptor
from sip.interfaces import INoopBreadcrumb
from sip.interfaces.sample import ISample
from sip.interfaces.location import ILocation
from sip.interfaces.aliquot import IAliquot
from sip.interfaces.adapter import ISampleWithAliquots, \
     IAugmentedSample, IZopeSample, IZopeAliquot, ISampleWithOriginAliquots
from sip.interfaces.source import IContainerSource, IContainerAliquotSource
from sip.services.servicenames import ProjectService
from sip.descriptors import BoundCallableDescriptor
from sip.schema.schema import BaseSchemaUtility
from sip.schema.fields import FieldsStorage
from sip.shipment.interfaces import IShipmentAliquots
from sip.utils import getPrincipals
from sip.interfaces.shippinginfo import IShippingWizard

class PrincipalRolesHelper:
    implements(IPrincipalRolesHelper)

    def __init__(self, context):
        self.context = context

    def availablePrincipals(self):
        principals = zapi.getService(Authentication,
                context=self.context).getPrincipals('')
        avail_principals = api.getRoot(self.context).available_principals
        principals = [p.id for p in principals
                      if p.getLogin() in avail_principals]
        return principals

    def availableRoles(self):
        avail = ['sip.AppManager',
                 'sip.ProjectManager',
                 'sip.LabTechnician',
                 'sip.Report',
                 'sip.User']

        roles = [name for name, role in zapi.getUtilitiesFor(IRole)
                 if name.startswith('sip.')]
        prinrole = IPrincipalRoleMap(self.context, None)
        prinroles = {}
        for principalid in getPrincipals():
            if prinrole is not None:
                for role, setting in prinrole.getRolesForPrincipal(principalid):
                    if setting in (Allow,):
                        prinroles[role] = setting
        # Roles the user has
        roles = [k for k, v in prinroles.items() if v in (Allow,)]
        # Filter against avail, to get it ordered
        roles = roles and filter(lambda x: x in roles, avail) or avail
        if roles:
            # Get the highest roles in the available
            # That the user is allowed to set
            roles = avail[avail.index(roles[0]):]
            for role in roles:
                setting = prinroles.get(role, Unset)
                # Then remove the ones he's denied
                if setting in (Deny,):
                    del roles[roles.index(role)]
        return roles


aug_unproxied = ('__Security_checker__', # To proxy the security settings
                                         # of the adapted object.
                 '_context',             # Internals
                 '_fields',
                 '__dict__',
                 '__class__',            # Expose our class
                 '__provides__',
                 '__implements__',       # Allow setting interface
                 '__providedBy__',       # declarations in the
                                         # adapter itself.
                 '__parent__',           # Containement.
                 '__name__',
                 '_reset')

class AugmentedSample:
    """This pretends to be an adapter, but it actually
    works more like a proxy.
    """
    implements(IAugmentedSample)

    __Security_checker__ = SecurityDescriptor(proxied_attr='_context')

    def __init__(self, context):
        self._context = context
        type_id = context.type_id
        self._reset(type_id)

    def _reset(self, type_id):
        context = self._context
        schema = api.getSchemaFor(type_id, context)
        factory = api.factory('sip.AdditionalSampleData', context=context)
        table = factory.q
        sample_id = context.id
        #st = api.getFactory(context).getFactory('SampleType')
        st = api.factory('sip.SampleType', context=context)
        stname = st.get(type_id).name
        self._fields = {}
        for key in schema.names():
            field_id = api.getFieldId(context, sample_id, key, stname)
            entries = factory.select(AND(table.sample_id == sample_id,
                                         table.schema_field_id == field_id))
            if entries.count():
                obj = entries[0]
                self._fields[key] = (obj, schema[key])
            else:
                self._fields[key] = (None, schema[key])

    def __getattribute__(self, name):
        if name in aug_unproxied:
            v = object.__getattribute__(self, name)
            # Handle descriptors here, eg: __Security_checker__
            if hasattr(v, '__get__'):
                # We should call it only if it came from the class,
                # otherwise its a descriptor being used as an instance
                # attribute, so just return it.
                if (hasattr(self, '__class__') and
                    getattr(self.__class__, name) is v):
                    return v.__get__(self, type(self))
            return v
        if name in self._fields:
            obj = self._fields[name][0]
            field = self._fields[name][1]
            if obj is not None:
                return obj.value
            else:
                return field.default
        proxied = self._context
        v = getattr(proxied, name)
        # And also handle descriptors for the proxied object,
        # but using the proxied object on __get__ calls.
        if hasattr(v, '__get__'):
            # We should call it only if it came from the class,
            # otherwise its a descriptor being used as an instance
            # attribute, so just return it.
            if (hasattr(proxied, '__class__') and
                getattr(proxied.__class__, name) is v):
                return v.__get__(proxied, type(proxied))
        return v

    def __setattr__(self, name, value):
        # Use a nested function to avoid security.  Need to do this to
        # preserve Sample security integrity when proxying attributes.
        def insert_field(name, value):
            context=self._context
            factory = api.factory('sip.AdditionalSampleData', context=context)
            stname = removeSecurityProxy(context).type_.name
            field_id = api.getFieldId(context, context.id,
                                      name, stname)
            value = str(value)
            field = factory(sample_id=context.id,
                            schema_field_id=field_id,
                            value=value)
            self._fields[name] = (field, self._fields[name][1])
        if name in aug_unproxied:
            object.__setattr__(self, name, value)
        elif name in self._fields:
            field = self._fields[name][1]
            field_type, value = api.convertFieldAndValue(field, value)
            field.validate(value)
            ad = self._fields[name][0]
            if ad:
                ad.value = value
                notify(ObjectModifiedEvent(ad))
            else:
                insert_field(name, value)
        else:
            setattr(self._context, name, value)
            notify(ObjectModifiedEvent(self._context))


class AliquotROContainer:
    implements(IReadContainer)


    def __init__(self, aliquots):
        aliquot_dict = {}
        for a in aliquots:
            aliquot_dict['Aliquot.%d' % a.id] = a
        self._aliquots = aliquot_dict

    def __getitem__(self, key):
        value = self._aliquots.get(key)
        if value is None:
            raise KeyError, key
        return value

    def get(self, key, default=None):
        value = self._aliquots.get(key, default)
        return value

    def __contains__(self, key):
        return key in self._aliquots

    def keys(self):
        return self._aliquots.keys()

    def __iter__(self):
        return self._aliquots.__iter__()

    def values(self):
        aliquots = self._aliquots
        return self._aliquots.values()

    def items(self):
        return self._aliquots.items()

    def __len__(self):
        return len(self._aliquots)


class SampleWithAliquots:
    implements(ISampleWithAliquots)

    def __init__(self, context):
        if ISample.providedBy(context):
            self._context = context
            factory = api.factory('sip.Aliquot', context=context)
            aliquots = factory.select(factory.q.sample_id == context.id)
            project = api.getProject(context)
            project = IReadContainer(project)
            alsource = project['aliquots']
            aliquots = [api.contained(a, alsource, 'sip.Aliquot.%d' % a.id)
                        for a in aliquots]
            self._aliquots = AliquotROContainer(aliquots)
        if IAliquot.providedBy(context):
            project = api.getProject(context)
            project = IReadContainer(project)
            ssource = project['samples']
            sample = context.sample_
            sample = self._context = api.contained(sample, ssource,
                                                   'sip.Sample.%d' % sample.id)
            self._aliquots = AliquotROContainer([context])

    def __getattr__(self, name):
        if name == 'aliquots':
            return self._aliquots
        return getattr(self._context, name)

    def __setattr__(self, name, value):
        if name.startswith('_'):
            self.__dict__[name] = value
            return
        if name != 'aliquots':
            setattr(self._context, name, value)
        else:
            raise AttributeError, "can't bind attribute"

    def update(self, other):
        new_aliquots = []
        for k, v in other.aliquots.items():
            if k not in self.aliquots:
                new_aliquots.append(v)
        old_aliquots = self.aliquots.values()
        self._aliquots = AliquotROContainer(new_aliquots + old_aliquots)


class SampleWithOriginAliquots:
    implements(ISampleWithOriginAliquots)

    def __init__(self, context):
        self._context = context
        if self.derived:
            factory = api.factory('sip.OriginSampleAliquots', context=self)
            aliquots = factory.select(factory.q.sample_id == context.id)
            project = api.getProject(context)
            project = IReadContainer(project)
            alsource = project['aliquots']
            aliquots = [api.contained(a.aliquot_, alsource,
                                      'sip.Aliquot.%d' % a.id)
                        for a in aliquots]
        else:
            aliquots = []
        self._origin_aliquots = AliquotROContainer(aliquots)

    def __getattr__(self, name):
        if name == 'origin_aliquots':
            return self._origin_aliquots
        return getattr(self._context, name)

    def __setattr__(self, name, value):
        if name.startswith('_'):
            self.__dict__[name] = value
            return
        if name != 'origin_aliquots':
            setattr(self._context, name, value)
        else:
            raise AttributeError, "can't bind attribute"


class RelationDescriptor:

    def __init__(self, container, sub, attr, prefix):
        self.container = container
        self.sub = sub
        self.attr = attr
        self.prefix = prefix

    def __get__(self, obj, objtype=None):
        func = getattr(api, 'get%s' % self.container)
        if not hasattr(obj, 'context'):
            return None
        ct = func(obj.context)
        value = getattr(obj.context, self.attr)
        if value is None:
            return None
        name = "%s.%s" % (self.prefix, value)
        return zapi.traverse(ct, self.sub)[name]

class ZopeAliquot:
    """ See ``.interfaces.adapter.IZopeAliquot`` """
    implements(IZopeAliquot)

    def __init__(self, context):
        self.context = context

    def __getattribute__(self, name):
        if name in ['context',
                    'volume_unit',
                    'concentration_unit',
                    'weight_unit',
                    'sample',
                    'container',
                    'project']:
            return object.__getattribute__(self, name)
        return getattr(self.context, name)

    volume_unit = RelationDescriptor('Root', 'volumeunits',
                                     'volume_id', 'sip.VolumeUnit')

    concentration_unit = RelationDescriptor('Root',
                                            'concentrationunits',
                                            'concentration_id',
                                            'sip.ConcentrationUnit')

    weight_unit = RelationDescriptor('Root', 'weightunits',
                                     'weight_id', 'sip.WeightUnit')

    project = RelationDescriptor('Root', 'projects',
                                 'project_id', 'sip.Project')

    sample = RelationDescriptor('Project', 'samples',
                                'sample_id', 'sip.Sample')

    def _get_container(self):
        path = ['locations']
        location = self.context.getLocation()
        location = [l['id'] for l in location]
        path.extend(location)
        path = filter(None, path)

        root = api.getRoot(self.context)
        return zapi.traverse(root, path)

    container = property(_get_container)

class ZopeSample:
    """ See ``.interfaces.adapter.IZopeSample`` """
    implements(IZopeSample)

    def __init__(self, context):
        self.context = context

    project = RelationDescriptor('Root', 'projects',
                                 'project_id', 'sip.Project')

    type = RelationDescriptor('Project', 'sampletypes',
                              'type_id', 'sip.SampleType')


class ProjectContainer(Contained):

    def __init__(self, context):
        self.context = context
        contained = api.contained(self, context, '.')

    def keys(self):
        return [k for k, v in self.items()]

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        return [v for k, v in self.items()]

    def items(self):
        ps = zapi.getService(ProjectService)
        return [(k, api.contained(v, self.context, name=k)) \
                for k, v in ps.queryContainers(self.context).items()]

    def __getitem__(self, name):
        ps = zapi.getService(ProjectService)
        item = ps.queryContainers(self.context).get(name)
        if item is None:
            raise KeyError, name
        return api.contained(item, self.context, name=name)

    def get(self, name, default=None):
        try:
            return self[name]
        except KeyError:
            return default

    def __contains__(self, name):
        return self.get(name, None) is not None

    def __len__(self):
        return len(self.keys())

class RootContainer(Contained):

    implements(ILocation, IContainerSource)

    def __init__(self, context):
        self.context = context
        contained = api.contained(self, context, '.')

    def getRootContainer(self):
        return self.context.getRootContainer()

    def getRootId(self):
        return self.context.getRootId()

    def __iter__(self):
        return iter(self.keys())

    def keys(self):
        return [k for k, v in self.items()]

    def values(self):
        return [v for k, v in self.items()]

    def items(self):
        root = self.context.getRootContainer()
        adapted = IReadContainer(root, None)
        return adapted and adapted.items() or []

    def __getitem__(self, name):
        root = self.context.getRootContainer()
        if not root:
            raise KeyError, name
        adapted = IReadContainer(root)
        return adapted[name]

    def get(self, name, default=None):
        try:
            return self[name]
        except KeyError:
            return default

    def __contains__(self, name):
        return self.get(name, None) is not None

    def __len__(self):
        return len(self.keys())

    def __delitem__(self, name):
        raise KeyError, "It's not possible to delete items. %s" % name

    def __setitem__(self, name, content):
        return name

class ContainerContainer(Contained):

    implements(IContainerAliquotSource)

    def __init__(self, context):
        self.context = context
        contained = api.contained(self, context, '.')

    def _get_id(self):
        return self.context.id

    id = property(_get_id)

    #def _get_location_id(self):
    #    location = api.getLocation(self.context, self.context)
    #    return location.location_id

    #location_id = property(_get_location_id)

    def getContainerQuery(self, id='', parentColumn='parent_id'):
        if id and parentColumn != 'container_id':
            query = '%s = %i and container_id = %s' % (
                parentColumn, self.id, id)
            return query
        query = '%s = %i' % (parentColumn, self.id)
        return query

    def keys(self):
        return [k for k, v in self.items()]

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        return [v for k, v in self.items()]

    def items(self, containers_only=False):
        context = self.context
        containerClassName = context.classNames[0]
        aliquotClassName = context.classNames[1]
        container_query = self.getContainerQuery()
        factory = api.factory(containerClassName, context=context)
        container_items = [(containerClassName+'.'+str(p.id),p) \
                           for p in factory.select(clause=container_query)]
        items = container_items
        if not containers_only:
            factory = api.factory(aliquotClassName, context=context)
            container_query = self.getContainerQuery(
                parentColumn='container_id')
            aliquot_items = [(aliquotClassName+'.'+str(p.id),p) \
                               for p in factory.select(clause=container_query)]
            items.extend(aliquot_items)
        return [(k, api.contained(v, context, name=k)) for k, v in items]

    def __getitem__(self, name):
        context = self.context
        try:
            parts = name.split('.')
            id = int(parts[-1])
            factoryName = '.'.join(parts[:-1])
        except ValueError:
            raise KeyError(name)
        if factoryName == 'sip.Aliquot':
            container_query = self.getContainerQuery(id, 'container_id')
        else:
            container_query = self.getContainerQuery(id)
        factory = api.factory(factoryName, context=context)
        results = factory.select(clause=container_query)

        if results.count() == 0:
            raise KeyError(name)
        return api.contained(results[0], context, name=name)

    def get(self, name, default=None):
        try:
            return self[name]
        except KeyError:
            return default

    def __contains__(self, name):
        return self.get(name, None) is not None

    def __len__(self):
        return len(self.keys())

    def __delitem__(self, name):
        obj = self[name]
        obj.destroySelf()

    def __setitem__(self, name, content):
        return name

class SQLSchema(BaseSchemaUtility, Contained):
    """An adapter to turn a Sample Type Schema into
    an IMutableSchema.
    """

    implements(INoopBreadcrumb)

    _attrs = BoundCallableDescriptor(FieldsStorage, call=True)

    def __init__(self, context):
        BaseSchemaUtility.__init__(self)
        self._context = context
        contained = api.contained(self, context, '.')

    def _get_id(self):
        return self._context.id

    id = property(_get_id)

    def getName(self):
        return self._context.name

    def _clear(self):
        self.schemaPermissions = {}

    def constrain(self, name, constraint):
        self._attrs.constrain(name, constraint)


class ShipmentAliquotsContainer(SampleContainer):

    implements(IShipmentAliquots)

    def __init__(self, context):
        self.__context = context
        super(ShipmentAliquotsContainer, self).__init__()

    def _newContainerData(self):
        aliquots = api.getAliquotsForShipment(self.__context)
        items = map(lambda x: (api.makeName(x), x), aliquots)
        return dict(api.containedItems(items, self.__context))

class Shipment:
    """ gather together the data for shipping wizards """

    implements(IShippingWizard)

    shipment_keys = ['name', 'shipdate', 'receivedate']
    contact_keys = ['firstname', 'lastname', 'institution', 'address1',
            'address2', 'city', 'state', 'zipcode', 'country', 'phone', 'fax',
            'email', 'project', 'objective']
    shipping_keys = ['hand_delivered', 'carrier_id', 'tracking_number',
            'instructions']

    def __init__(self, context):
        self.context = self.shipment = removeSecurityProxy(context)
        self.contact_info = removeSecurityProxy(context).contact_info_
        self.shipping_info = removeSecurityProxy(context).shipping_info_
        self.aliquots = ShipmentAliquotsContainer(context)

    def __getattribute__(self, name):
        context = self
        if name in ['shipment', 'contact_info',
                    'context', 'shipping_info',
                    'shipment_keys', 'shipping_keys',
                    'contact_keys', 'aliquots']:
            return object.__getattribute__(self, name)
        elif name in self.shipment_keys:
            context = self.shipment
        elif name in self.shipping_keys:
            context = self.shipping_info
        elif name in self.contact_keys:
            context = self.contact_info
        return object.__getattribute__(context, name)

    def __setattr__(self, name, value):
        context = None
        if name in self.shipment_keys:
            context = self.shipment
        elif name in self.shipping_keys:
            context = self.shipping_info
        elif name in self.contact_keys:
            context = self.contact_info
        else:
            object.__setattr__(self, name, value)
            notify(ObjectModifiedEvent(self))
            return
        setattr(context, name, value)
        notify(ObjectModifiedEvent(context))
