# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: aliquot.py 759 2005-06-28 23:45:42Z sidnei $
"""

__metaclass__ = type

from xml.sax.saxutils import escape

from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent
from zope.app import zapi
from zope.app.form import CustomWidgetFactory
from zope.app.form.browser import IntWidget, DisplayWidget
from zope.app.form.browser import ChoiceInputWidget
from zope.app.form.browser.submit import Update

from sip.browser import api
from sip.browser.widget import HiddenWidget, JSHelperWidget
from sip.browser.widget import ROIntWidget
from sip.aliquot import Aliquot

class IntDisplayWidget(DisplayWidget):

    def __call__(self):
        if self._renderedValueSet():
            return escape(unicode(self._data))
        else:
            return escape(unicode(self.context.default))

class EditAliquot:

    def _setUpWidgets(self):
        context = self.context
        self.id_widget = CustomWidgetFactory(IntDisplayWidget)
        self.volume_id_widget = CustomWidgetFactory(ChoiceInputWidget,
                                             size=1)
        self.weight_id_widget = CustomWidgetFactory(ChoiceInputWidget,
                                             size=1)
        self.concentration_id_widget = CustomWidgetFactory(ChoiceInputWidget,
                                                    size=1)

        project_name = 'sip.Project.%s' % context.project_id
        project = api.getRoot(context)['projects'][project_name]
        locations = zapi.traverse(project, 'locationsproject')
        url = str(zapi.getView(locations, 'absolute_url', self.request))
        url += '/@@container_tree.html'
        kw = {}

        if 'SAMPLE_EDIT_FORM' in self.request:
            kw['img_url'] = \
                  '?aliquot_id=%s&SAMPLE_EDIT_FORM=1#aliquot_edit' % context.id
            self.sample_id_widget = CustomWidgetFactory(HiddenWidget)

        kw['img_extra'] = """onclick='window.open("%s","")'""" % url
        kw['img_src'] = '/++skin++sip/@@/sip_img/ico_add_sample.gif'
        kw['img_alt'] = 'Select one'
        kw['img_title'] = 'Select one'
        kw['img_class'] = 'container'
        self.container_id_widget = JSHelperWidget(IntWidget, **kw)


        return super(EditAliquot, self)._setUpWidgets()


class AddAliquot:

    def create(self, *args, **kw):
        self.sample_id = kw.get('sample_id')
        if not 'project_id' in kw:
            factory = api.factory('sip.Sample')
            sample = factory.get(self.sample_id)
            kw['project_id'] = sample.project_id
        return super(AddAliquot, self).create(*args, **kw)


    def _setUpWidgets(self):
        # XXX We shouldn't depend on the Aliquot class
        # for getting the nextID. Replace it with an adapter.
        self.id_widget = CustomWidgetFactory(ROIntWidget,
                                      value=Aliquot.nextID(self.context))
        self.volume_id_widget = CustomWidgetFactory(ChoiceInputWidget,
                                             size=1)
        self.weight_id_widget = CustomWidgetFactory(ChoiceInputWidget,
                                             size=1)
        self.concentration_id_widget = CustomWidgetFactory(ChoiceInputWidget,
                                                    size=1)
        if 'SAMPLE_EDIT_FORM' in self.request:
            self.sample_id_widget = CustomWidgetFactory(HiddenWidget)
        container = api.queryContainer(self.context)
        if container is not None:
            self.request.form['field.container_id'] = container.id
            self.container_id_widget = CustomWidgetFactory(HiddenWidget)
        else:
            project = api.getProject(self.context)
            locations = zapi.traverse(project, 'locationsproject')
            url = str(zapi.getView(locations, 'absolute_url', self.request))
            url += '/@@container_tree.html'
            kw = {}
            if 'SAMPLE_EDIT_FORM' in self.request:
                kw['img_url'] = '#aliquot'
            kw['img_extra'] = """onclick='window.open("%s","")'""" % url
            kw['img_src'] = '/++skin++sip/@@/sip_img/ico_add_sample.gif'
            kw['img_alt'] = 'Select one'
            kw['img_title'] = 'Select one'
            kw['img_class'] = 'container'
            self.container_id_widget = JSHelperWidget(IntWidget, **kw)
        return super(AddAliquot, self)._setUpWidgets()

    def nextURL(self):
        if self.sample_id is not None:
            project = api.getProject(self.context)
            sample = zapi.traverse(project,
                                   'samples/sip.Sample.%s' % self.sample_id)
            url = zapi.getView(sample, 'absolute_url', self.request)()
            return "%s#aliquot" % url
        return super(AddAliquot, self).nextURL()


class AliquotComment:

    def getComment(self):
        comment = getattr(self.context, 'comment', '')
        return comment
