# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: app_setup.py 749 2005-06-28 22:21:49Z sidnei $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.zapi import traverse
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app.publisher.browser import BrowserView
from sip.browser.admin import NextURLProxy
from sip.browser import api

class AppSetupView(BrowserView):

    def __init__(self, context, request):
        super(AppSetupView, self).__init__(context, request)
        self.setupViews()

    def setupViews(self):
        context = self.context
        request = self.request
        self.views = {}
        self.forms = {}
        projects = traverse(context, 'projects')
        adding = zapi.getView(projects, '+', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, context)
        project_add = zapi.getView(adding, 'add_project.html', request)
        project_add.setPrefix('project')
        self.views['project'] = NextURLProxy(project_add, self)
        self.forms['project'] = project_add.generated_form
        sampletypes = traverse(context, 'sampletypes')
        adding = zapi.getView(sampletypes, '+', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, context)
        sampletype_add = zapi.getView(adding, 'add_sample_type.html', request)
        sampletype_add.setPrefix('sampletype')
        self.views['sampletype'] = NextURLProxy(sampletype_add, self)
        self.forms['sampletype'] = sampletype_add.generated_form
        locations = traverse(context, 'locations')
        adding = zapi.getView(locations, '+', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, context)
        location_add = zapi.getView(adding, 'add_location.html', request)
        location_add.setPrefix('location')
        self.views['location'] = NextURLProxy(location_add, self)
        self.forms['location'] = location_add.generated_form
        cunits = traverse(context, 'concentrationunits')
        adding = zapi.getView(cunits, '+', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, context)
        cunit_add = zapi.getView(adding, 'add_concentration_unit.html', request)
        self.views['cunit'] = NextURLProxy(cunit_add, self)
        self.forms['cunit'] = cunit_add.generated_form
        wunits = traverse(context, 'weightunits')
        adding = zapi.getView(wunits, '+', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, context)
        wunit_add = zapi.getView(adding, 'add_weight_unit.html', request)
        self.views['wunit'] = NextURLProxy(wunit_add, self)
        self.forms['wunit'] = wunit_add.generated_form
        vunits = traverse(context, 'volumeunits')
        adding = zapi.getView(vunits, '+', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, context)
        vunit_add = zapi.getView(adding, 'add_volume_unit.html', request)
        self.views['vunit'] = NextURLProxy(vunit_add, self)
        self.forms['vunit'] = vunit_add.generated_form

        containertypes = traverse(context, 'containertypes')
        adding = zapi.getView(containertypes, '+', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, context)
        containertype_add = zapi.getView(adding, 'add_container_type.html',
                                         request)
        containertype_add.setPrefix('containertype')
        self.views['containertype'] = NextURLProxy(containertype_add, self)
        self.forms['containertype'] = containertype_add.generated_form
        avail_projects = zapi.getView(context, 'projects.html', request)
        self.views['avail_projects'] = NextURLProxy(avail_projects, self)
        self.forms['avail_projects'] = avail_projects.generated_form
        avail_locations = zapi.getView(context, 'locations.html', request)
        self.views['avail_locations'] = NextURLProxy(avail_locations, self)
        self.forms['avail_locations'] = avail_locations.generated_form
        avail_containertypes = \
                          zapi.getView(context, 'containertypes.html', request)
        self.views['avail_containertypes'] = \
                                       NextURLProxy(avail_containertypes, self)
        self.forms['avail_containertypes'] = \
                                            avail_containertypes.generated_form
        avail_wunits = zapi.getView(context, 'weightunits.html', request)
        self.views['avail_wunits'] = NextURLProxy(avail_wunits, self)
        self.forms['avail_wunits'] = avail_wunits.generated_form
        avail_vunits = zapi.getView(context, 'volumeunits.html', request)
        self.views['avail_vunits'] = NextURLProxy(avail_vunits, self)
        self.forms['avail_vunits'] = avail_vunits.generated_form
        avail_cunits = zapi.getView(context, 'concentrationunits.html', request)
        self.views['avail_cunits'] = NextURLProxy(avail_cunits, self)
        self.forms['avail_cunits'] = avail_wunits.generated_form
        avail_st = zapi.getView(context,
                                'avail_st.html', request)
        self.views['avail_sampletypes'] = NextURLProxy(avail_st, self)
        self.forms['avail_sampletypes'] = avail_st.generated_form


    def update(self):
        view = None
        if 'ADD_PROJECT' in self.request:
            view = 'project'
        elif 'ADD_SAMPLE_TYPE' in self.request:
            view = 'sampletype'
        elif 'ADD_LOCATION' in self.request:
            view = 'location'
        elif 'ADD_UNIT' in self.request:
            unit_type = self.request.get('unit_type')
            view = unit_type
        elif 'ADD_CONTAINERTYPE' in self.request:
            view = 'containertype'
        elif 'ADMIN_PROJECT' in self.request:
            view = self.views['avail_projects'].getView()
            p_id = view.available_projects_widget.getInputValue()
            project = traverse(self.context,
                               'projects/%s' % p_id)
            url = zapi.getView(project, 'absolute_url', self.request)()
            self.request.response.redirect(url + '/@@admin_project.html')
            return
        elif 'ADMIN_LOCATION' in self.request:
            view = self.views['avail_locations'].getView()
            l_id = view.available_locations_widget.getInputValue()
            location = traverse(self.context,
                                'locations/sip.Location.%s' % l_id)
            url = zapi.getView(location, 'absolute_url', self.request)()
            self.request.response.redirect(url + '/@@edit.html')
            return
        elif 'ADMIN_CONTAINERTYPE' in self.request:
            view = self.views['avail_containertypes'].getView()
            ct_id = view.available_containertypes_widget.getInputValue()
            containertype = traverse(self.context,
                                     'containertypes/%s' % ct_id)
            url = zapi.getView(containertype, 'absolute_url', self.request)()
            self.request.response.redirect(url + '/@@edit.html')
            return
        elif 'MANAGE_CUNITS' in self.request:
            view = self.views['avail_cunits'].getView()
            u_id = view.available_concentrationunits_widget.getInputValue()
            unit = traverse(self.context,
                        'concentrationunits/sip.ConcentrationUnit.%s' % u_id)
            url = zapi.getView(unit, 'absolute_url',self.request)()
            self.request.response.redirect(url + '/@@edit.html')
            return
        elif 'MANAGE_WUNITS' in self.request:
            view = self.views['avail_wunits'].getView()
            u_id = view.available_weightunits_widget.getInputValue()
            unit = traverse(self.context,
                              'weightunits/sip.WeightUnit.%s' % u_id)
            url = zapi.getView(unit, 'absolute_url', self.request)()
            self.request.response.redirect(url + '/@@edit.html')
            return
        elif 'MANAGE_VUNITS' in self.request:
            view = self.views['avail_vunits'].getView()
            u_id = view.available_volumeunits_widget.getInputValue()
            unit = traverse(self.context,
                            'volumeunits/sip.VolumeUnit.%s' % u_id)
            url = zapi.getView(unit, 'absolute_url',self.request)()
            self.request.response.redirect(url + '/@@edit.html')
            return
        if view:
            view = self.views[view].getView()
            if not view.update_status:
                status = view.update()
                self.setupViews()
                return status

    app_setup = ViewPageTemplateFile('app_setup.pt')

    def nextURL(self):
        return (str(zapi.getView(self.context, "absolute_url", self.request))
                + '/@@app_setup.html')
