# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

from zope.app.publisher.browser import BrowserView
from sip.browser import api
from sip.browser.shippinginfo import getkwds
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent, ObjectCreatedEvent
from zope.app.datetimeutils import parseDatetimetz
from zope.app.datetimeutils import DateTimeError
from datetime import datetime
from sip.i18n import _

funding_attributes = ('grant_number', 'cost', 'agency',
                      'principal', 'state',
                      'fee', 'enddate')

def hasData(d):
    if d['funding_info_id']:
        return True
    for k in funding_attributes:
        if d[k]:
            return True
    return False

def toDict(obj):
    d = {}
    keys = []
    for c in obj._columns:
        k = c.kw.get('name')
        if k is None:
            # Changed in sqlobject 0.6.x
            k = getattr(c, '_name')
        keys.append(k)
    for k in keys:
        d[k] = getattr(obj,k)
    return d

class FundingInfoView(BrowserView):

    update_status = None

    def _getInfo(self, federal=True):
        context = self.context
        request = self.request
        factory = api.factory('sip.FundingSource', context=context)
        sources = factory.selectBy(federal=federal)
        if self.errors:
            data = request['data']
            data_map = {}
            for d in data:
                d = dict(d)
                d['funding_source_id'] = sid = int(d['funding_source_id'])
                data_map[sid] = d
            info = []
            for s in sources:
                data_map[s.id]['name'] = s.name
                info.append(data_map[s.id])
            return info
        funding_sources = {}
        for source in sources:
            funding_sources[source.id] = toDict(source)
            funding_sources[source.id]['funding_source_id'] = source.id
        factory = api.factory('sip.FundingInfo', context=context)
        info = factory.selectBy(shipment_id=context.id)
        funding_info = {}
        kw = getkwds(factory)
        del kw['funding_source_id']
        for i in info:
            source_id = i.funding_source_id
            funding_info[source_id] = toDict(i)
            funding_info[source_id]['funding_info_id'] = i.id
        for k, source in funding_sources.items():
            if k in funding_info:
                funding_info[k]['cost'] = format_cost(funding_info[k]['cost'])
                funding_info[k]['fee'] = format_cost(funding_info[k]['fee'])
                source.update(funding_info[k])
                source['show_commit'] = 1
            else:
                source.update(kw)
                source['show_commit'] = 0
            source['shipment_id'] = context.id
        funding_info = funding_sources.items()
        funding_info.sort()
        funding_info = [x[1] for x in funding_info]
        return funding_info


    def info(self):
        self.shipment_id = self.context.id
        info = [{'label': 'Federal', 'sources': self._getInfo(federal=True)},
                {'label': 'Non Federal',
                 'sources': self._getInfo(federal=False)}]
        return info

    def _dataChanged(self, data):
        context = self.context
        factory = api.factory('sip.FundingInfo', context=self.context)
        id = data['funding_info_id']
        if not id: return False
        info = factory.get(int(id))
        for k in funding_attributes:
            v = getattr(info, k)
            if v is not None and data[k] != str(v): return True
        return False


    def _setInfo(self, data):
        factory = api.factory('sip.FundingInfo', context=self.context)
        for d in data:
            del d['show_commit']
            id = d['funding_info_id']
            del d['funding_info_id']
            d['shipment_id'] = int(d['shipment_id'])
            d['funding_source_id'] = int(d['funding_source_id'])
            if id:
                id = int(id)
                info = factory.get(id)
                for k,v in d.items():
                    setattr(info, k, v)
                notify(ObjectModifiedEvent(info))
            else:
                d['commit_message'] = 'created'
                info = factory(**d)
                notify(ObjectCreatedEvent(info))

    def update(self):
        shipment = self.context
        self.errors = errors = {}
        request = self.request
        if self.update_status is not None:
            return self.update_status
        status = ''
        if 'UPDATE_SUBMIT' in request:
            project = api.queryProject(self.context)
            msg = ''
            if project is not None and not project.require_commit_message:
                principal = request.principal
                principal_id = principal and principal.getLogin() \
                        or '<unknown user>'
                msg = u"Modified by '%s'" % principal_id
            data = request['data']
            processed = []
            dataChanged = self._dataChanged
            for d in data:
                hasdata = hasData(d)
                d = dict(d)
                if d['funding_info_id'] and not dataChanged(d):
                    continue
                source_id = int(d['funding_source_id'])
                if msg:
                    d['commit_message'] = msg
                if not d['show_commit']:
                    d['commit_message'] = 'created'
                if not d.get('commit_message', '').strip() and hasdata:
                    error = getError(errors, source_id)
                    error['commit_message'] = 'input required'
                    status = _("An error occured.")
                for k,v in d.items():
                    v = str(v).strip()
                    if not v: v = None
                    if k in ('cost', 'fee') and v is not None:
                        try:
                            v = remove_format(v)
                            v = float(v)
                        except ValueError:
                            error = getError(errors, source_id)
                            error[k] = '%s is not a float' % v
                            status = _("An error occured.")
                    elif k == 'enddate' and v is not None:
                        try:
                            v =  parseDatetimetz(v).date()
                            requestdate = shipment.requestdate
                            if requestdate and v < requestdate:
                                error = getError(errors, source_id)
                                error['enddate'] = 'enddate < request date'
                                status = _("An error occured.")
                        except (DateTimeError, ValueError, IndexError):
                            error = getError(errors, source_id)
                            error['enddate'] = '%s is not a valid date' % v
                            status = _("An error occured.")

                    d[k] = v
                if hasdata:
                    processed.append(d)
            if len(processed) and not self.errors:
                self._setInfo(processed)
                formatter = self.request.locale.dates.getFormatter(
                    'dateTime', 'medium')
                status = _("Updated on ${date_time}")
                status.mapping = {'date_time': formatter.format(
                    datetime.utcnow())}
            self.update_status = status

        return self.update_status

    def showCommit(self):
        project = api.queryProject(self.context)
        return project is not None and project.require_commit_message


def getError(errors, source_id):
    if not errors.has_key(source_id):
        errors[source_id] = error = {}
    else:
        error = errors[source_id]
    return error

def format_cost(cost):
    if not cost: return cost
    cost = str(cost)
    parts = cost.split('.')
    if len(parts) == 2:
        cost = parts[0]
        change = parts[1]
    else:
        change = ''
    length = len(cost)
    if length <= 3:
        if change:
            cost = '.'.join((cost, change))
        return cost
    result = ''
    while length >= 3:
        if len(result):
            result = ','.join((cost[-3:], result))
        else:
            result = cost[-3:]
        cost = cost[:-3]
        length = len(cost)
    if length:
        result = ','.join((cost, result))
    if change:
        result = '.'.join((result, change))
    return result

def remove_format(cost):
    parts = cost.split(',')
    cost = ''.join(parts)
    return cost
