# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: search.py 759 2005-06-28 23:45:42Z sidnei $
"""

import hotshot, time
from datetime import datetime
from zope.app import zapi
from zope.app.publisher.browser import BrowserView
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app.datetimeutils import parse, DateError, SyntaxError, DateTimeError

from sip import File
from sip.browser import api
from sip.query import QueryCommand, SearchType, Query, SortOn
from sip.csv.utils import dump_samples
from sip.interfaces.utilities import IQuery

def _extractAliquotInfo(self, aliquot):
    a = {}
    root = api.getRoot(self.context)
    root_url = str(zapi.getView(root, 'absolute_url', self.request))
    concentration = aliquot.concentration
    sample = aliquot.sample_ # ForeignKey == column_name[:-2]
    a['id'] = aliquot.id
    a['concentration'] = aliquot.getConcentrationString()
    a['volume'] = aliquot.getVolumeString()
    a['weight'] = aliquot.getWeightString()
    a['label'] = aliquot.label
    a['aliquot_count'] = aliquot.aliquot_count
    a['location'] = aliquot.getLocation()
    a['container_type'] = aliquot.getContainerType() or '&nbsp;'
    a['container_id'] = aliquot.container_id
    a['url'] = ('%s/projects/sip.Project.%d/samples/sip.Sample.%d'
                '?aliquot_id=%d&SAMPLE_EDIT_FORM=1#aliquot_edit'
                % (root_url, sample.project_id, sample.id, aliquot.id))
    a['aliquot_url'] = ('%s/projects/sip.Project.%d/aliquots/sip.Aliquot.%d'
                        % (root_url, sample.project_id, aliquot.id))
    a['comment'] = aliquot.comment
    if aliquot.is_low:
        a['is_low'] = 'yes'
    else:
        a['is_low'] = 'no'

    if aliquot.depleted:
        a['depleted'] = 'yes'
    else:
        a['depleted'] = 'no'
    return a

class AliquotQuery(BrowserView):

    modify_aliquot = File('csv/templates/modify_aliquot.csv')
    derive_sample = File('csv/templates/derive_sample.csv')

    has_aliquots = False

    def getSampleTypes(self):
        sampletypes = zapi.traverse(self.context, 'sampletypes')
        stypes = sampletypes.values()
        dicts = []
        for stype in stypes:
            d = {}
            d['id'] = stype.id
            d['name'] = stype.name
            dicts.append(d)
        return dicts

    def getFields(self):
        request = self.request
        context = self.context
        project_id = context.id
        sample_type_id = request.get('sample_type')
        if not sample_type_id:
            sample_type_id = self.getSampleTypes()[0]['id']
        results = api.getFields(self.context, project_id, sample_type_id)
        fields = []
        index = 0
        for r in results:
            dict = {}
            dict['field_id'] = r.schema_field_id
            dict['title'] = r.title or r.name
            if r.type in ('zope.schema._bootstrapfields.TextLine',
                          'zope.schema._bootstrapfields.Text',
                          'zope.schema._bootstrapfields.Bool'):
                dict['ops'] = ('=',)
            else:
                dict['ops'] = ('<=', '=', '>=')
            if r.type == 'zope.schema._bootstrapfields.Bool':
                dict['value_tag'] = (
                  "<select name='info_querys.value:records' size='1'>\n"
                  "  <option value=''>No Value</option>\n"
                  "  <option value='True'>True</option>\n"
                  "  <option value='False'>False</option>\n"
                  "</select>\n")
            elif r.type == 'zope.schema._field.Date':
                id = 'datevalue%i' % (index, )
                index += 1
                value_tag = (
                  '<input name="info_querys.value:records"\n'
                  '       id="%s"\n'
                  '       type="text"\n'
                  '       />\n'
                  '<a href="javascript:show_calendar(%r);">\n'
                  '<img src="/@@/show-calendar.gif"\n'
                  '     width=24 height=22 border=0></a>\n')
                dict['value_tag'] = value_tag % (id, id)
            else:
                dict['value_tag'] = (
                  "<input name='info_querys.value:records'\n"
                  "       type='text'\n"
                  "       />\n")
            fields.append(dict)
        return fields

    def getProjects(self):
        root = api.queryRoot(self.context)
        projects = zapi.traverse(root, 'projects')
        projects = projects.values()
        dicts = []
        for project in projects:
            d = {}
            d['id'] = project.id
            d['name'] = project.name
            dicts.append(d)
        return dicts

    def getOrderbys(self):
        return [{'id':'sampleid',
                 'label':'Sample ID'},
                {'id':'dategen',
                 'label':'Date Generated'}]

    def getName(self):
        return self.context.name

    def getAliquots(self):
        return getattr(self,'aliquots',[])

    def export_csv(self):
        request = self.request
        request.form['search'] = 'search'
        alids = [str(a) for a in request.get('aliquot_ids', [])]
        request.form['aliquot_ids'] = '\n'.join(alids)
        response = request.response
        response.setHeader('content-type', 'application/x-csv')
        response.setHeader('content-disposition',
                               'attachment; filename=modify_aliquot.csv')
        self.doSearch()
        return str(self.modify_aliquot) % dump_samples(
            self.results.getSamples())

    def splitorderive(self):
        request = self.request
        response = request.response
        alids = request.get('aliquot_ids')
        def set_fname(fname):
            response.setHeader('content-type', 'application/x-csv')
            response.setHeader('content-disposition',
                               'attachment; filename=%s' % fname)
        view = zapi.getView(self.context, 'search_results.html',
                            request)
        if not alids and request.get('derive'):
            request.form['message'] = 'You must select one or more aliquots'+\
                       'Click on the back button and try again'
            return view()

        if request.get('split'):
            aliquots = request.get('aliquots')
            try:
                aliquots = [a for a in aliquots
                            if a['split_count'] and int(a['split_count'])]
            except ValueError:
                request.form['message'] = 'Split counts must be integers' + \
                          'Click on the back button and try again'
                return view()
            if not len(aliquots):
                request.form['message'] =\
                   'You must enter a split count for at least one aliquot.' + \
                   'Click on the back button and try again'
                return view()
            request.form['aliquots'] = aliquots
            if request.get('export_csv'):
                set_fname('split_aliquot.csv')
                view = zapi.getView(self.context, 'split_aliquot.csv', request)
            else:
                view = zapi.getView(self.context, 'aliquot_split.html',
                                    request)
            return view()

        if request.get('export_csv'):
            set_fname('derive_sample.csv')
            view = zapi.getView(self.context, 'derive_sample.csv', request)
            return view()

        view = zapi.getView(self.context, 'derive_aliquot_edit.html', request)
        return view()

    def validate(self):
        request = self.request
        collect_dates = request.get('collect_dates')
        entered_dates = request.get('entered_dates')
        if collect_dates:
            message = dates_validation(collect_dates)
            if message:
                message = 'Date Collected: %s ' % message

            entered_message = dates_validation(entered_dates)
            if entered_message:
                message = '%sDate Entered: %s' % (message, entered_message)
            if message.strip():
                request.form['message'] = message
                view = zapi.getView(self.context, 'search.html',
                                    request)
                return view()
        if request.get('ADDITIONAL_QUERY') and not request.get('search'):
            view = zapi.getView(self.context, 'additional_search.html',
                                request)
            sample_types = request.get('sample_types', [])
            if len(sample_types):
                request.form['sample_type'] = sample_types[0]
            return view()
        labels = request.get('aliquot_ids')
        if labels:
            labels = labels.strip()
        if labels:
            valid = idValidation(labels)
            if not valid:
                request.form['message'] = 'Aliquot IDs must be integers'
                view = zapi.getView(self.context, 'search.html',
                                    request)
                return view()
        sample_ids = request.get('sample_ids')
        if sample_ids:
            valid = idValidation(sample_ids)
            if not valid:
                request.form['message'] = 'Sample IDs must be integers'
                view = zapi.getView(self.context, 'search.html',
                                    request)
                return view()
        if request.get('export_to_csv'):
            view = zapi.getView(self.context, 'dump.csv', request)
            return view()
        view = zapi.getView(self.context, 'search_results.html',
                            request)
        return view()


    def doSearch(self):
        context = self.context
        request = self.request
        if 'search' not in request:
            return []
        cmds = []
        qs = zapi.getUtility(IQuery, context=context)
        project_ids = [context.id]
        query_dict = {}
        all_project_ids = [p['id'] for p in self.getProjects()]

        if 'project_ids' in request:
            projids = request['project_ids']
            if projids:
                project_ids = projids
        query_dict['project_ids'] = project_ids

        if request.get('aliquot_ids'):
            labels = request['aliquot_ids']
            if labels:
                labels = str(labels).strip()
                labels = labels.splitlines()
                cmds.append(QueryCommand(SearchType.aliquot, labels=labels,
                                     project_ids=all_project_ids))

        if request.get('sample_ids'):
            ids = request['sample_ids']
            if ids:
                ids = str(ids).strip()
                ids = ids.splitlines()
                cmds.append(QueryCommand(SearchType.sampleid,
                            sample_ids = ids, project_ids=all_project_ids))

        if request.get('names'):
            names = request['names']
            if names:
                names = str(names).strip()
                names = names.splitlines()
                cmds.append(QueryCommand(SearchType.samplename,
                            names = names, project_ids=all_project_ids))

        if request.get('other_ids'):
            ids = request['other_ids']
            if ids:
                ids = str(ids).strip()
                ids = ids.splitlines()
                cmds.append(QueryCommand(SearchType.otherid, ids = ids,
                                     project_ids=all_project_ids))

        if request.get('subject_ids'):
            ids = request['subject_ids']
            if ids:
                ids = str(ids).strip()
                ids = ids.splitlines()
                query_dict['subject_ids'] = ids
                query_dict['id_type'] = request.get('id_type')

        if request.get('sample_types'):
            ids = request['sample_types']
            if ids:
                query_dict['type_ids'] = ids

        if request.get('collect_dates'):
            query_dict['collect_dates'] = request['collect_dates']

        if request.get('entered_dates'):
            query_dict['entered_dates'] = request['entered_dates']

        if request.get('info_querys'):
            query_dict['info_querys'] = request['info_querys']
            cmds.append(QueryCommand(SearchType.addinfo, **query_dict))
        else:
            cmds.append(QueryCommand(SearchType.project, **query_dict))

        if request.get('order_by'):
            order_by = getattr(SortOn, request['order_by'])
        else:
            order_by = None



        query = Query(cmds, sorton=order_by)
        results = qs.search(query)
        self.results = results

    def dump(self):
        return self.export_csv()

    def getResults(self):
        request = self.request
        results = []
        if not hasattr(self, 'results'):
            return []
        # We modify results in place here.
        # This was done so we can cache the permission
        # lookups for projects, instead of doing it
        # all over and over again for the same project.
        self._extractSampleInfo(results)
        return results

    _extractAliquotInfo = _extractAliquotInfo

    def _extractSampleInfo(self, results):
        has_aliquots = False
        p_perms = {}

        empty = lambda a: not a.isEmpty()
        root = api.queryRoot(self.context)
        root_url = str(zapi.getView(root, 'absolute_url', self.request))
        projects = root['projects']
        extractAliquotInfo = self._extractAliquotInfo
        for result in self.results.getSamples():
            sample = result
            r = {}
            results.append(r)

            r['sample'] = s = {}
            aliquots = filter(empty, result.aliquots.values())
            if aliquots: has_aliquots=True
            r['aliquots'] = map(self._extractAliquotInfo, aliquots)
            s['name'] = sample.name
            s['other_id'] = sample.other_id
            s['subject_id'] = sample.subject_id
            s['id'] = sample.id
            s['date_generated'] = sample.date_generated
            s['collection_date'] = sample.collection_date

            # Try to cache permissions per project, so they
            # are looked up fewer times. Permissions are very
            # unlikely to change middle-request anyway.

            perm = p_perms.get(sample.project_id, None)
            if perm is None:
                project_id=sample.project_id
                project = projects['sip.Project.%d' % project_id]
                perm = {'edit':
                        api.checkSIPPermission(project, 'sip.EditSample'),
                        'add':
                        api.checkSIPPermission(project, 'sip.AddSample')
                        }
                p_perms[project_id] = perm

            s['can_edit'] = perm['edit']
            s['can_add'] = perm['add']

            s['sample_type'] = sample.type_.name
            s['url'] = '%s/projects/sip.Project.%d/samples/sip.Sample.%d'\
                       % (root_url, sample.project_id, sample.id)
        self.has_aliquots = has_aliquots

        return results

    def canDerive(self):
        return self.has_aliquots and api.checkSIPPermission(self.context,
                'sip.AddSample')

    def canSplit(self):
        return self.has_aliquots and api.checkSIPPermission(self.context,
                'sip.AddAliquot')

    def canImport(self):
        return api.checkSIPPermission(self.context, 'sip.CSVImport')

    index = ViewPageTemplateFile('search.pt')
    query_results = ViewPageTemplateFile('search_results.pt')

def dates_validation(dates):
    start_date = dates['start_date']
    end_date = dates['end_date']
    message = ''
    test_start = test_end = None
    try:
        if start_date.strip():
            test_start = parse(start_date)
            test_start = datetime(test_start[0], test_start[1], test_start[2])
    except (DateError, DateTimeError, SyntaxError):
        message = ' Start Date invalid: %s' % start_date
    try:
        if end_date.strip():
            test_end = parse(end_date)
            test_end = datetime(test_end[0], test_end[1], test_end[2])
    except (DateError, DateTimeError, SyntaxError):
        message += ' End Date invalid: %s' % end_date
    if test_start and test_end:
        if test_start > test_end:
            message = ' End Date must be greater than Start Date'
    return message

def idValidation( ids ):
    ids = ids.splitlines()
    for id in ids:
        try:
            int(id.strip())
        except ValueError:
            return False
    return True
