# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: shippinginfo.py 759 2005-06-28 23:45:42Z sidnei $
"""

__metaclass__ = type

from sip.browser import api
from sip.browser.container import AddContainer
from zope.app.form.browser.editview import EditView
from zope.app import zapi
from zope.event import notify
from zope.app.event.objectevent import ObjectCreatedEvent, ObjectModifiedEvent
from zope.app.form.utility import setUpWidgets, getWidgetsData
from zope.app.form.interfaces import WidgetsError, WidgetInputError
from sip.interfaces.shippinginfo import IShippingInfo, IContactInfo
from zope.app.form.utility import setUpEditWidgets, applyWidgetsChanges
from zope.security.proxy import removeSecurityProxy
from zope.schema.interfaces import ValidationError
from sip.shipment.interfaces import IShipment
from sip.i18n import _
from datetime import datetime
from zope.schema import getFieldNamesInOrder

def getkwds(factory):
    kw = {}
    for c in factory._columns:
        try:
            kw[c.kw['name']] = None
        except KeyError:
            kw[c.name] = None
    return kw


class InfoBase(EditView):

    fieldNames = property(lambda self: self.__dict__.get('fieldNames', None)
                                or getFieldNamesInOrder(self.schema),
        lambda self,names: self._setFieldNames(names))

    def _setFieldNames(self, names):
        self.__dict__['fieldNames'] = names

    def __init__(self, *args, **kw):
        super(InfoBase, self).__init__(*args, **kw)
        project = api.queryProject(self.context)
        if self.info is None or project is not None \
                             and not project.require_commit_message:
            filter_out = ['commit_message']
            f = lambda x: x not in filter_out
            self.fieldNames = filter(f, self.fieldNames)

    def _setUpWidgets(self):
        setUpEditWidgets(self, self.schema, source=self.info,
                names=self.fieldNames)

    def update(self):
        if self.update_status is not None:
            return self.update_status
        status = ''
        request = self.request
        content = self.info
        project = api.queryProject(self.context)
        created = False
        if 'UPDATE_SUBMIT' in self.request and project is not None:
            if not project.require_commit_message:
                principal = request.principal
                principal_id = principal and principal.getLogin() or \
                        '<unknown user>'
                msg = u"Modified by '%s'" % principal_id
                request.form['field.commit_message'] = msg
                names = tuple(self.fieldNames) + ('commit_message',)
            elif content is None:
                created = True
                request.form['field.commit_message'] = 'new'
                names = tuple(self.fieldNames) + ('commit_message',)
            else:
                names = tuple(self.fieldNames)
            changed = True
            try:
                if not content:
                    data = getWidgetsData(self, self.schema,
                            names=names)
                    self.info = content = self.factory(**data)
                    notify(ObjectCreatedEvent(content))
                    self.setInfo(content)
                else:
                    changed = applyWidgetsChanges(self, self.schema,
                            target=content, names=names)
                    if changed:
                        notify(ObjectModifiedEvent(content))
            except WidgetsError, errors:
                self.errors = errors
                status = _("An error occured.")
            else:
                if created:
                    request.response.redirect(request.getURL())
                    return
                setUpEditWidgets(self, self.schema, source=content,
                        ignoreStickyValues=True,
                        names=self.fieldNames)
                if changed:
                    formatter = self.request.locale.dates.getFormatter(
                        'dateTime', 'medium')
                    status = _("Updated on ${date_time}")
                    status.mapping = {'date_time': formatter.format(
                        datetime.utcnow())}
            self.update_status = status
        return status

class ShippingInfoView(InfoBase):

    schema = IShippingInfo

    def __init__(self, context, request):
        context = removeSecurityProxy(context)
        self.info = context.shipping_info_
        self.factory = api.factory('sip.ShippingInfo', context=context)
        super(ShippingInfoView, self).__init__(context, request)

    def setInfo(self, info):
        shipment = self.context
        shipment.shipping_info_id = info.id
        notify(ObjectModifiedEvent(shipment))


class ContactInfoView(InfoBase):

    schema = IContactInfo

    def __init__(self, context, request):
        context = removeSecurityProxy(context)
        self.info = context.contact_info_
        self.factory = api.factory('sip.ContactInfo', context=context)
        super(ContactInfoView, self).__init__(context, request)

    def setInfo(self, info):
        shipment = self.context
        shipment.contact_info_id = info.id
        notify(ObjectModifiedEvent(shipment))

class AddShipment:

    def update(self):
        if self.update_status is not None:
            # We've been called before. Just return the previous result.
            return self.update_status
        if 'UPDATE_SUBMIT' in self.request:
            self.update_status = ''
            try:
                data = getWidgetsData(self, self.schema, names=self.fieldNames)
                obj = self.createAndAdd(data)
            except WidgetsError, errors:
                self.errors = errors
                self.update_status = _("An error occured.")
                return self.update_status
            except ValidationError, v:
                widget = self.receivedate_widget
                error = WidgetInputError(widget.name, widget.label, v)
                widget._error = error
                self.errors = (error,)
                self.update_status = _("An error occured.")
                return self.update_status
            self.request.response.redirect(self.nextURL())
        return self.update_status

    def create(self, *args, **kw):
        skw = {'project_id': api.getProject(self.context).id,
                'name': kw['name'],
                'requestdate': kw['requestdate'],
                'planneddate': kw['planneddate'],
                'shipdate': kw['shipdate'],
                'receivedate': kw['receivedate'],
                'returndate': kw['returndate'],
                'infectious': kw.get('infectious', False),
                'commit_message': kw.get('commit_message'),
                'biohazard': kw.get('biohazard', False),}
        factory = api.factory('sip.Shipment', context=self.context)
        kw = getkwds(factory)
        kw.update(skw)
        shipment = factory(**kw)
        IShipment.validateInvariants(shipment)
        notify(ObjectCreatedEvent(shipment))
        factory = api.factory('sip.Container', context=self.context)
        parent = zapi.getParent(self.context)
        kw = getkwds(factory)
        kw['parent_id'] = 1
        kw.update({'name':'Shipment %d' % shipment.id,
                  'commit_message':'Shipment Container'})
        container = factory(**kw)
        notify(ObjectCreatedEvent(container))
        factory = api.factory('sip.ContainerShipment', context=self.context)
        cs = factory(**{'container_id': container.id,
                        'shipment_id': shipment.id})
        notify(ObjectCreatedEvent(cs))
        return shipment

    def nextURL(self):
        shipments = zapi.traverse(api.getProject(self.context), 'shipments')
        url = str(zapi.getView(shipments, 'absolute_url', self.request))
        return url+'/@@dashboard.html'

class EditShipment:

    def update(self):
        if self.update_status is not None:
            # We've been called before. Just return the status we previously
            # computed.
            return self.update_status
        status = ''
        content = self.adapted
        request = self.request
        if 'UPDATE_SUBMIT' in self.request:
            project = api.queryProject(self.context)
            if project is not None and not project.require_commit_message:
                principal = request.principal
                principal_id = principal and principal.getLogin() or \
                        '<unknown user>'
                msg = u"Modified by '%s'" % principal_id
                request.form['field.commit_message'] = msg
            changed = False
            try:
                changed = applyWidgetsChanges(self, self.schema,
                    target=content, names=self.fieldNames)
                # We should not generate events when an adapter is used.
                # That's the adapter's job.
                if changed and self.context is self.adapted:
                    IShipment.validateInvariants(content)
                    notify(ObjectModifiedEvent(content))
            except WidgetsError, errors:
                self.errors = errors
                status = _("An error occured.")
            except ValidationError, v:
                widget = self.receivedate_widget
                error = WidgetInputError(widget.name, widget.label, v)
                widget._error = error
                self.errors = (error,)
                self.update_status = _("An error occured.")
            else:
                setUpEditWidgets(self, self.schema, source=self.adapted,
                                 ignoreStickyValues=True,
                                 names=self.fieldNames)
                if changed:
                    self.changed()
                    formatter = self.request.locale.dates.getFormatter(
                        'dateTime', 'medium')
                    status = _("Updated on ${date_time}")
                    status.mapping = {'date_time': formatter.format(
                        datetime.utcnow())}
        self.update_status = status
        return status
