# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: splitderive.py 758 2005-06-28 23:02:39Z sidnei $
"""

from zope.app import zapi
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app.publisher.browser import BrowserView

from sip.browser import api
from sip.browser.search import _extractAliquotInfo
from sip.browser.admin import NextURLProxy
from sip.interfaces.utilities import ISplitDerive
from sip.interfaces.adapter import IZopeAliquot
from sip import File
from sip.csv.utils import dump_aliquots
from zope.app.container.interfaces import IReadContainer

def fromSameSample(aliquots):
    sample_id = aliquots[0].sample_id
    for aliquot in aliquots:
        if aliquot.sample_id != sample_id:
            return False
    return True

def testValues(rec):
    if hasattr(rec, 'depleted') or rec.has_key('depleted') and rec['depleted']:
        return ()
    for k in ('weight', 'volume', 'concentration'):
        value = rec[k]
        if not value:
            continue
        try:
            value = float(value)
        except ValueError:
            return (k, value)
        if value < 0:
            return (k, str(value))
    k = 'container_id'
    value = rec[k]
    if not value:
        return ()
    try:
        value = int(value)
    except ValueError:
        return (k, value)
    if value <= 0:
        return (k, str(value))
    return ()

def getUnits(self):
    context = self.context
    units = {'weight':[],'volume':[],'concentration':[]}
    factory = api.factory('sip.WeightUnit', context=context)
    res = factory.select()
    if res.count():
        units['weight'] = [{'label':u.label, 'id':u.id} for u in res]
    factory = api.factory('sip.VolumeUnit', context=context)
    res = factory.select()
    if res.count():
        units['volume'] = [{'label':u.label, 'id':u.id} for u in res]
    factory = api.factory('sip.ConcentrationUnit', context=context)
    res = factory.select()
    if res.count():
        units['concentration'] =\
                              [{'label':u.label, 'id':u.id} for u in res]
    return units

class SplitCSV(BrowserView):

    split_template = File('csv/templates/split_aliquot.csv')

    def _getSourceAliquots(self):
        request = self.request
        context = self.context
        project = api.getProject(context)
        as = zapi.traverse(project, 'aliquots')
        factory = api.factory('sip.Aliquot', context=context)
        aliquots = []
        for split in request['aliquots']:
            if not split['split_count']:
                continue
            id = int(split['id'])
            try:
                count = abs(int(split['split_count']))
            except ValueError:
                continue
            aliquots.append({'aliquot':api.contained(factory.get(id), as),
                            'count':count})
        return aliquots

    def _getNewAliquots(self, aliquots):
        request = self.request
        context = self.context
        out = ('project_name,aliquot_id,source_aliquot_id,sample_id,'
               'sample_name,weight,weight_units,volume,'
               'volume_units,concentration,concentration_units,chiclet_count,'
               'is_low,depleted,container_path,comment\r\n')
        template = '%s,%i,%i,%i,%s,,,,,,,,,,%s\r\n,'
        for a in aliquots:
            aliquot = IZopeAliquot(a['aliquot'])
            count = a['count']
            source_id = aliquot.id
            project_name = aliquot.project.name
            sample_name = aliquot.sample.name
            sample_id = aliquot.sample.id
            new_ids = api.allocateAliquotIds(context, count)
            path = aliquot.getLocation()
            if path is not None:
                names = [d['title'].strip() for d in path]
                path = "path://%s" % '/'.join(names)
            path = path or ''
            for id in new_ids:
                out += template % (project_name, id, source_id, sample_id,
                    sample_name, path)
        return out

    def getCSV(self):
        source_aliquots = self._getSourceAliquots()
        aliquots = [a['aliquot'] for a in source_aliquots]
        data = dump_aliquots(aliquots)
        data['new_aliquots'] = self._getNewAliquots(source_aliquots)
        return str(self.split_template) % data


class AliquotSplit(BrowserView):

    def finishSplit(self):
        request = self.request
        context = self.context
        factory = api.factory('sip.Aliquot', context=context)
        info = []
        project = api.getProject(context)
        project = IReadContainer(project)
        aliquot_source = project['aliquots']
        for split in request['aliquots']:
            if not split['split_count']:
                continue
            id = int(split['id'])
            try:
                count = int(split['split_count'])
            except ValueError:
                request.form['error_msg'] = \
                     'Illegal input for split count: %s' % split['split_count']
                return
            if count < 0:
                request.form['error_msg'] = \
                      'Illegal input for split count: %d' % count
                return
            clause = 'aliquot_id = %d' % id
            res = factory.select(clause = clause)
            aliquot = res[0]
            aliquot = api.contained(aliquot, aliquot_source,
                'Aliquot.%d' % aliquot.id)
            ainfo = _extractAliquotInfo(self, aliquot)
            ainfo['new_ids'] = api.allocateAliquotIds(context, count)
            for k in ('weight', 'volume', 'concentration'):
                if ainfo[k] != '&nbsp;':
                    value, unit = ainfo[k].split(' ')
                    ainfo[k] = value
                    ainfo[k + 'unit'] = unit
                else:
                    ainfo[k] = ''
                    ainfo[k + 'unit'] = ''
            info.append(ainfo)
        self.aliquots = info

    def editAliquots(self):
        request = self.request
        context = self.context
        base_aliquots = request['base_aliquots']
        new_aliquots = request['aliquots']
        for rec in base_aliquots:
            test = testValues(rec)
            if test:
                request.form['error_msg'] = \
                  'Illegal input for %s: %s' % test
                return []
        for rec in  new_aliquots:
            test = testValues(rec)
            if test:
                request.form['error_msg'] = \
                  'Illegal input for %s: %s' % test
                return []
        ss = zapi.getUtility(ISplitDerive, context=context)
        aliquots = ss.split(base_aliquots, new_aliquots)
        return [_extractAliquotInfo(self, a) for a in aliquots]

    getUnits = getUnits

    def getAliquots(self):
        return getattr(self, 'aliquots', [])



    aliquot_split = ViewPageTemplateFile('aliquot_split.pt')
    split_results = ViewPageTemplateFile('split_results.pt')

class DeriveAliquotEdit(BrowserView):

    getUnits = getUnits

    def getAliquots(self):
        request = self.request
        context = self.context
        aliquot_ids = request.get('aliquot_ids')
        aliquots = []
        factory = api.factory('sip.Aliquot', context=context)
        as = IReadContainer(context)['aliquots']
        for id in aliquot_ids:
            clause = 'aliquot_id = %d' % id
            res = factory.select(clause = clause)
            al = res[0]
            aliquot = api.contained(al, as, 'Aliquot.%d'%al.id)
            ainfo = _extractAliquotInfo(self, aliquot)
            for k in ('weight', 'volume', 'concentration'):
                if ainfo[k] != '&nbsp;':
                    value, unit = ainfo[k].split(' ')
                    ainfo[k] = value
                    ainfo[k + 'unit'] = unit
                else:
                    ainfo[k] = ''
                    ainfo[k + 'unit'] = ''
            aliquots.append(ainfo)
        return aliquots

    def editAliquots(self):
        request = self.request
        context = self.context
        aliquots = request.get('aliquots')
        aliquot_ids = []
        factory = api.factory('sip.Aliquot', context=context)
        for rec in aliquots:
            id = int(rec['id'])
            aliquot_ids.append(id)
            res = factory.select(clause = "aliquot_id = %d" % id)
            aliquot = res[0]
            if rec['aliquot_count']:
                aliquot.aliquot_count = int(rec['aliquot_count'])
            else:
                aliquot.aliquot_count = None

            if (hasattr(rec, 'is_low') or rec.has_key('is_low')) \
                                         and rec['is_low']:
                aliquot.is_low = True
            else:
                aliquot.is_low = False

            if (hasattr(rec, 'depleted') or rec.has_key('depleted')) \
                                         and rec['depleted']:
                aliquot.depleted = True
            else:
                aliquot.depleted = False
            comment = rec['comment']
            if comment:
                aliquot.comment = comment
            values = {}
            for k in ('weight', 'volume', 'concentration'):
                value = rec[k]
                if value:
                    values[k] = value
                    setattr(aliquot, k, None)
                unit = k + '_unit'
                if hasattr(rec, unit):
                    setattr(aliquot, unit, int(rec[unit]))
            notify(ObjectModifiedEvent(aliquot))
            if values:
                for k,v in values.items():
                    setattr(aliquot, k, float(v))
                notify(ObjectModifiedEvent(aliquot))
            if rec['container_id']:
                aliquot.container_id = int(rec['container_id'])
        request.form['aliquot_ids'] = aliquot_ids
        view = zapi.getView(context, 'aliquot_derive.html', request)
        return view()


class SampleDeriveCSV(BrowserView):

    sample_derive = File('csv/templates/derive_sample.csv')

    def _getSourceAliquots(self):
        context = self.context
        request = self.request
        alids = request.get('aliquot_ids')
        project = api.getProject(context)
        factory = api.factory('sip.Aliquot', context=context)
        as = zapi.traverse(project, 'aliquots')
        aliquots = [IZopeAliquot(api.contained(factory.get(id), as,id))
                    for id in alids]
        return aliquots

    def _getDerivedSample(self, aliquots):
        context = self.context
        out = ('project_name,sample_name,sample_type,source_aliquot_id(s),'
              'subject_id,subject_type,other_id,technician,collection_date,'
              'comments\r\n')
        alids = [str(a.id) for a in aliquots]
        id_str = '"%s"' % ','.join(alids)
        if fromSameSample(aliquots):
            template = '%s,,,%s,%s,%s,%s,,%s,\r\n'
            sample = aliquots[0].sample
            subject_type = api.getSubjectType(context, sample.subject_type_id)
            project_name = aliquots[0].project.name
            out += template % (project_name, id_str, sample.subject_id,
                               subject_type, sample.other_id,
                               sample.collection_date)
        else:
            template = ',,,%s,,,,,,\r\n'
            out += template % id_str
        return out

    def getCSV(self):
        source_aliquots = self._getSourceAliquots()

        data = dump_aliquots(source_aliquots)
        data['derived_sample'] = self._getDerivedSample(source_aliquots)
        return str(self.sample_derive) % data

class AliquotDerive(BrowserView):

    def __init__(self, context, request):
        self.views = {}
        self.forms = {}
        self.request = request
        self.context = context
        if 'aliquot_ids' not in request:
            request.form['error_msg'] = 'You forgot to select aliquots'
            return
        alids = request['aliquot_ids']
        factory = api.factory('sip.Aliquot', context=context)
        if len(alids) == 1:
            clause = 'aliquot_id = %d' % alids[0]
        else:
            clause = 'aliquot_id in %s' % str(tuple(alids))
        aliquots = factory.select(clause = clause)
        if fromSameSample(aliquots):
            sample = aliquots[0].sample_
            factory = api.factory('sip.SubjectType', context=context)
            st = factory.get(sample.subject_type_id)
            request.form['subject_type_id'] = st.name
            request.form['other_id'] = sample.other_id
            request.form['subject_id'] = sample.subject_id
            request.form['collection_date'] = sample.collection_date
        samples = zapi.traverse(context, 'samples')
        adding = zapi.getView(samples, '+', request)
        adding = api.contained(adding, samples)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, samples)
        sample_add = zapi.getView(adding, 'DerivedSample', request)
        self.views['derive'] = NextURLProxy(sample_add, self)
        self.forms['derive'] = sample_add.generated_form

    def update(self):
        view = None
        request = self.request
        if 'DERIVE' in request:
            view = 'derive'
        if view:
            view = self.views[view].getView()
            if not view.update_status:
                view.update()


    aliquot_derive = ViewPageTemplateFile('aliquot_derive.pt')
