# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: tree_adapters.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import implements

from zope.app import zapi
from zope.app.tree.interfaces import IChildObjects
from zope.app.container.interfaces import IReadContainer
from zope.proxy import removeAllProxies

from sip.interfaces.location import ILocation
from sip.interfaces.container import IContainer
from sip.shipment.interfaces import IShipment, IOutgoingShipmentState
from sip.interfaces.source import ILocationProjectSource
from sip.browser import api

class BaseChildObjects:
    def __init__(self, context):
        self.context = context

    def hasChildren(self):
        children = IReadContainer(self.context)
        values = children.values()
        for obj in values:
            if (IContainer.providedBy(obj) or
                ILocation.providedBy(obj) or
                IShipment.providedBy(obj) or
                IOutgoingShipmentState.providedBy(obj)):
                return True
        return False

class LocationProjectSourceChildObjects(BaseChildObjects):
    implements(IChildObjects)
    __used_for__ = ILocationProjectSource

    def getChildObjects(self):
        project = api.getProject(self.context)
        locations = zapi.traverse(project, 'locations')
        children = IReadContainer(locations)
        values = list(children.values())
        shipments = zapi.traverse(project, 'shipments/outgoing/incomplete')
        values.append(shipments)
        return values

class ContainerChildObjects(BaseChildObjects):
    implements(IChildObjects)
    __used_for__ = IContainer

    def getChildObjects(self):
        children = IReadContainer(self.context)
        return children.values()

class LocationChildObjects(ContainerChildObjects):
    implements(IChildObjects)
    __used_for__ = ILocation

class ShipmentChildObjects(ContainerChildObjects):
    implements(IChildObjects)
    __used_for__ = IShipment
