# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: metadirectives.py 749 2005-06-28 22:21:49Z sidnei $
"""
from zope.configuration.fields import GlobalObject
from zope.interface import Interface
from zope.schema import TextLine
from sip.interfaces.csv import IProcessor, ITempStorage, IMassager

class IBaseCSVComponentDirective(Interface):
    """This directive registers a component for a CSV file."""

    filename = TextLine(
        title=u"Filename",
        description=(u"This is the filename for the CSV file "
                     u"this component will be bound to."),
        required=True)

    component = GlobalObject(
        title=u"Component",
        description=(u"Specifies the component that will act "
                     u"upon the given file."),
        required=True)

class IProvideProcessorDirective(IBaseCSVComponentDirective):
    """Provides a processor component"""

    interface = GlobalObject(
        title=u"The interface this component provides.",
        description=(u"Components provide an interface, which "
                     u"is then used to register and query through "
                     u"the utility service."),
        required=True,
        default=IProcessor,
        )

class IProvideStorageDirective(IBaseCSVComponentDirective):
    """Provides a storage component"""

    interface = GlobalObject(
        title=u"The interface this component provides.",
        description=(u"Components provide an interface, which "
                     u"is then used to register and query through "
                     u"the utility service."),
        required=True,
        default=ITempStorage,
        )

class IProvideMassagerDirective(IBaseCSVComponentDirective):
    """Provides a massager component"""

    interface = GlobalObject(
        title=u"The interface this component provides.",
        description=(u"Components provide an interface, which "
                     u"is then used to register and query through "
                     u"the utility service."),
        required=True,
        default=IMassager,
        )
