# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: sample.py 759 2005-06-28 23:45:42Z sidnei $
"""

__metaclass__ = type

import traceback

from sqlobject import AND

from zope.app import zapi
from zope.security.proxy import ProxyFactory
from zope.exceptions import NotFoundError
from zope.app.rdb.interfaces import DatabaseException
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent

from sip.csv.processor.common import Processor
from sip.browser import api
from sip.interfaces.utilities import ISplitDerive
from sip.interfaces.adapter import IAugmentedSample

class AddSampleAliquot(Processor):
    """Processes ``add_sample_aliquot.csv``, creating samples
    and their respective aliquots when available"""

    def __init__(self, filename):
        self._results = {}
        super(AddSampleAliquot, self).__init__(filename)

    def process(self, context, data, request):
        "do actual work."
        stats = {'sample':0,
                 'additional_data':0,
                 'aliquots':0,
                 'errors':0}
        project = api.queryProject(context)
        message = request.form.get('commit_message')
        if not message:
            if request.principal:
                message = "Added by '%s'" % request.principal.id
            else:
                message = "(Empty commit message)"
        d_samples = data.get('sample', None)
        form = request.form
        sample_factory = api.factory('sip.Sample', context=context)
        aliquot_factory = api.factory('sip.Aliquot', context=context)
        ad = data.get('additional_data', None)
        sample_ad = {}
        if ad:
            for sample_data in ad:
                d = sample_ad.setdefault(sample_data['sample_name'], {})
                d[sample_data['field_title_or_id']] = sample_data['field_value']
            del d
        del ad
        d_aliquots = data.get('aliquots', None)
        sample_aliquots = {}
        if d_aliquots:
            for aliquot in d_aliquots:
                al = sample_aliquots.setdefault(aliquot['sample_name'], [])
                al.append(aliquot)
        del d_aliquots
        if d_samples:
            for sample in d_samples:
                sample_name = sample['name'] = sample['sample_name']
                sample = self._create_sample(sample, project, message,
                                             request, stats, sample_factory)
                if sample is None:
                    continue
                if sample_ad.get(sample_name):
                    self._set_additional_sample_data(sample, project,
                                                     sample_ad, sample_name,
                                                     stats)
                if sample_aliquots.get(sample_name):
                    for aliquot in sample_aliquots[sample_name]:
                        self._create_aliquot(aliquot, sample, project, request,
                                             message, aliquot_factory, stats)
        for res in self._results.values():
            print >> self.log, res['sample']
            for ad in res['additional_data']:
                print >> self.log, '  ' + ad
            for al in res['aliquots']:
                print >> self.log, '  ' + al
        self._results = {}
        ## if sample_ad:
        ##     edit_samples_that_already_existed()
        ## if sample_aliquots:
        ##     create_aliquots_for_samples_that_already_existed()
        return stats

    def _set_additional_sample_data(self, sample, project, sample_ad,
                                    sample_name, stats,
                                    stat_name='additional_data'):
        ad_results = self._results[sample_name]['additional_data']
        ad = {}
        schema = api.getSchemaFor(sample.type_id, project)
        fields = {}
        for name in schema.names():
            field = schema[name]
            fields[field.title] = field.__name__
            fields[field.__name__] = field.__name__
        for name, value in sample_ad[sample_name].items():
            try:
                setattr(sample, fields[name], value)
            except Exception, exc:
                stats['errors'] += 1
                ad_results.append('An error occurred while '
                                    'trying to set field %s '
                                    'of sample %s to %r.' %
                                    (name, sample_name, value))
                traceback.print_exc(file=self.log)
                if isinstance(exc, DatabaseException):
                    raise
                continue
            else:
                stats[stat_name] += 1
                ad_results.append('Sucessfully set '
                                    'field %s of sample '
                                    '%s to %s.' %
                                    (name, sample_name, value))

    def _create_sample(self, sample, project, message, request,
                       stats, sample_factory, stat_name='sample'):
        sample_name = sample['sample_name']
        self._results[sample_name] = {'additional_data':[],'aliquots':[]}
        del sample['sample_name']
        if sample['name'].lower().startswith('$s'):
            sample['name'] = None
        if not sample['collection_date']:
            sample['collection_date'] = None
        sample['project_id'] = project.id
        if not sample.get('commit_message'):
            sample['commit_message'] = message
        try:
            sample = sample_factory(**sample)
        except Exception, exc:
            stats['errors'] += 1
            self._results[sample_name]['sample'] = (
                                'An error occurred while trying '
                                'to add a new sample.')
            traceback.print_exc(file=self.log)
            if isinstance(exc, DatabaseException):
                raise
            return
        else:
            stats[stat_name] += 1
            self._results[sample_name]['sample'] = (
                                'Added new sample: %s (%s).  ' %
                                (sample.id, sample.name))
        sample = zapi.traverse(project, 'samples/sip.Sample.%s' % sample.id)
        return IAugmentedSample(sample)

    def _create_aliquot(self, aliquot, sample, project, request,
                        message, aliquot_factory, stats, stat_name='aliquots'):
        sample_name = aliquot['sample_name']
        del aliquot['sample_name']
        aliquot['sample_id'] = sample.id
        aliquot['project_id'] = project.id
        aliquot['id'] = api.allocateAliquotIds(project)[0]
        if not aliquot.get('comment'):
            aliquot['comment'] = None
        values = {}
        # make sure we set values after units
        for k in ('concentration', 'weight', 'volume'):
            value = aliquot[k]
            if value is not None:
                values[k] = value
                aliquot[k] = None
        if not aliquot.has_key('commit_message'):
            aliquot['commit_message'] = (message)
        try:
            aliquot = aliquot_factory(**aliquot)
            if values:
                for k,v in values.items():
                    setattr(aliquot, k, v)
                notify(ObjectModifiedEvent(aliquot))
        except Exception, exc:
            stats['errors'] += 1
            self._results[sample_name]['aliquots'].append(
                                'An error occurred while trying '
                                'to add a new aliquot.')
            traceback.print_exc(file=self.log)
            if isinstance(exc, DatabaseException):
                raise
        else:
            stats[stat_name] += 1
            self._results[sample_name]['aliquots'].append(
                                'Added new aliquot: %s (%s).  ' %
                                (aliquot.id, aliquot.name))

def fromSameSample(context, aliquot_ids):
    aliquots = []
    aliquot_factory = api.factory('sip.Aliquot', context=context)
    aliquots = [aliquot_factory.get(int(id)) for id in aliquot_ids]
    sample_id = aliquots[0].sample_id
    for aliquot in aliquots:
        if aliquot.sample_id != sample_id:
            return False
    return True

class DeriveSample(AddSampleAliquot):

    def process(self, context, data, request):
        "do actual work."
        stats = {'derived_aliquots':0,
                 'source_aliquots':0,
                 'derived_samples_additional_data':0,
                 'derived_samples':0,
                 'errors':0}
        root = api.queryRoot(context)
        message = request.form.get('commit_message')
        if not message:
            if request.principal:
                message = "Added by '%s'" % request.principal.id
            else:
                message = "(Empty commit message)"
        form = request.form
        sample_factory = api.factory('sip.Sample', context=context)
        aliquot_factory = api.factory('sip.Aliquot', context=context)
        # modify source_aliquots
        s_aliquots = data.get('source_aliquots', ())
        source_aliquots = {}
        for aliquot_d in s_aliquots:
            if 'sample_name' in aliquot_d:
                del aliquot_d['sample_name']
            aliquot_d['sample_id'] = int(aliquot_d['sample_id'])
            aliquot = self._modifyAliquot(aliquot_d, message, stats,
                                          aliquot_factory,
                                          stat_name='source_aliquots')
            if aliquot:
                source_aliquots[aliquot.id] = aliquot

        d_samples = data.get('derived_samples', ())

        ad = data.get('derived_samples_additional_data', ())
        sample_ad = {}
        for sample_data in ad:
            d = sample_ad.setdefault(sample_data['sample_name'], {})
            d[sample_data['field_title_or_id']] = sample_data['field_value']
        if len(ad): del d
        del ad
        d_aliquots = data.get('derived_aliquots', ())
        sample_aliquots = {}
        for aliquot in d_aliquots:
            al = sample_aliquots.setdefault(aliquot['sample_name'], [])
            al.append(aliquot)
        del d_aliquots
        for sample in d_samples:
            sample_name = sample['name'] = sample['sample_name']
            project = zapi.traverse(root, 'projects/sip.Project.%s' %
                                    sample['project_id'])
            sample['derived'] = 1
            aliquot_ids = sample['source_aliquot_id(s)']
            del sample['source_aliquot_id(s)']
            if not aliquot_ids:
                stats['errors'] += 1
                print >> self.log, ('An error has occured when trying to '
                                    'derive a sample. Source aliquot ids '
                                    'not found.')
                continue
            aliquot_ids = aliquot_ids.split(',')
            try:
                aliquot_ids = [int(a.strip()) for a in aliquot_ids]
            except ValueError:
                stats['errors'] += 1
                print >> self.log, ('An error has occured when trying to '
                                    'derive a sample. Source aliquot ids '
                                    'must be integers')
                continue
            if len(aliquot_ids) == 1 or fromSameSample(context,aliquot_ids):
                orig_aliquot = source_aliquots.get(aliquot_ids[0], None)
                if orig_aliquot is None:
                    orig_aliquot = aliquot_factory.get(int(aliquot_ids[0]))

                orig_sample = orig_aliquot.sample_
                keys = ('collection_date', 'other_id', 'subject_id',
                        'subject_type_id')
                for k in keys:
                    value = getattr(orig_sample, k)
                    if value:
                        sample[k] = value
            sample = self._create_sample(sample, project, message,
                                         request, stats,
                                         sample_factory,
                                         stat_name='derived_samples')
            if sample is None:
                continue
            ds = zapi.getUtility(ISplitDerive, context=context)
            ds.derive(sample.id, aliquot_ids)
            self._results[sample_name]['origin_ids'] = aliquot_ids

            if sample_ad.get(sample_name):
                self._set_additional_sample_data(
                    sample, project, sample_ad, sample_name,
                    stats, stat_name='derived_samples_additional_data')
            if sample_aliquots.get(sample_name):
                for aliquot in sample_aliquots[sample_name]:
                    project = zapi.traverse(root,
                        'projects/sip.Project.%s' % aliquot['project_id'])
                    self._create_aliquot(aliquot, sample, project, request,
                                         message, aliquot_factory, stats,
                                         stat_name='derived_aliquots')

        for res in self._results.values():
            print >> self.log, res['sample']
            for ad in res['additional_data']:
                print >> self.log, '  ' + ad
            for al in res['aliquots']:
                print >> self.log, '  ' + al


        return stats

    def _modifyAliquot(self, aliquot_d, message, stats,
                       aliquot_factory, stat_name='aliquots'):
        try:
            aliquot_id = aliquot_d['aliquot_id']
            del aliquot_d['aliquot_id']
            if not aliquot_d.get('commit_message'):
                aliquot_d['commit_message'] = message
            if aliquot_id:
                aliquot = aliquot_factory.get(int(aliquot_id))
                unchanged = True
                for k in aliquot_d.keys():
                    if (getattr(aliquot, k) != aliquot_d.get(k) and
                        k != 'commit_message'):
                        unchanged = False
                        break
                if unchanged:
                    log_msg = 'Aliquot %s unchanged'
                    print >> self.log, (log_msg % aliquot.id)
                    return aliquot
                values = {}
                for k in ('concentration', 'volume', 'weight'):
                    value = aliquot_d[k]
                    if value is not None:
                        values[k] = value
                        aliquot_d[k] = None

                aliquot.set(**aliquot_d)
                aliquot = api.uncontained(aliquot)
                notify(ObjectModifiedEvent(aliquot))
                if values:
                    for k,v in values.items():
                        setattr(aliquot, k, v)
                    notify(ObjectModifiedEvent(aliquot))
                log_msg = 'Successfully edited aliquot %s.'
        except Exception, exc:
            stats['errors'] += 1
            print >> self.log, ('An error occurred while trying '
                                'to edit aliquot %s.'
                                % aliquot_id)
            traceback.print_exc(file=self.log)
            if isinstance(exc, DatabaseException):
                raise
            return None
        else:
            stats[stat_name] += 1
            print >> self.log, (log_msg % aliquot.id)
            return aliquot
