# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/



from zope.app import zapi
from sip.browser import api
import unittest
from sip.ftests.base import Base
from datetime import date
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent

class TestShipment(Base):
    level = 2
    use_test_user = False

    def setUp(self):
        super(TestShipment, self).setUp()
        self.project = project = self._addProject(name='test_project')
        self.incomplete = zapi.traverse(project,
                'shipments/outgoing/incomplete')
        self.intransit = zapi.traverse(project, 'shipments/outgoing/intransit')
        self.received = zapi.traverse(project, 'shipments/outgoing/received')

    def testShipmentStates(self):
        project = self.project
        incomplete = self.incomplete
        intransit = self.intransit
        received = self.received
        for s in [incomplete, intransit, received]:
            self.failIf(len(s) != 0, 'len(%s) is %d' % (s.name, len(s)))
        shipment = self._addShipment(project, name='test_shipment')
        self.failIf(len(incomplete) != 1)
        self.failIf(len(intransit) != 0 and len(received) != 0)
        shipment.shipdate = date.today()
        notify(ObjectModifiedEvent(shipment))
        self.failIf(len(intransit) != 1)
        self.failIf(len(incomplete) != 0 and len(received) != 0)
        shipment.receivedate = date.today()
        notify(ObjectModifiedEvent(shipment))
        self.failIf(len(received) != 1)
        self.failIf(len(incomplete) != 0 and len(intransit) != 0)

    def testFundingInfo(self):
        project = self.project
        shipment = self._addShipment(project, name='test_shipment')
        source_factory = api.factory('sip.FundingSource', context=project)
        info_factory = api.factory('sip.FundingInfo', context=project)
        res = info_factory.selectBy(shipment_id=shipment.id)
        self.failIf(res.count() != 0)
        view = zapi.getView(shipment, 'funding.html', self.request)
        self.failUnless(view())
        sources = source_factory.select()
        source = sources[0]
        d = {}
        d['funding_source_id'] = source.id
        d['shipment_id'] = shipment.id
        d['funding_info_id'] = ''
        d['grant_number'] = 'qx12345'
        d['cost'] = '16234.0'
        d['agency'] = 'The Agency'
        d['principal'] = 'Bob Dobbs'
        d['state'] = 'Texas'
        d['enddate'] = ''
        d['fee'] = ''
        d['show_commit'] = 0
        view.request.form['data'] = [d]
        view.request.form['UPDATE_SUBMIT'] = True
        view()

        res = info_factory.selectBy(shipment_id=shipment.id)

        self.failIf(res.count() != 1)
        self.failIf(res[0].funding_source_id != source.id)
        self.failIf(res[0].shipment_id != shipment.id)
        self.failIf(res[0].grant_number != 'qx12345')

    def testContactInfo(self):
        project = self.project
        request = self.request
        shipment = self._addShipment(project, name='test_shipment')
        factory = api.factory('sip.ContactInfo', context=project)
        res = factory.select()
        self.failIf(res.count() != 0)
        view = zapi.getView(shipment, 'contact.html', request)
        self.failUnless(view())
        view.request.form['field.firstname'] = 'Bob'
        view.request.form['field.lastname'] = 'Dobbs'
        view.request.form['field.institution'] = 'Slackers Inc.'
        view.request.form['field.address1'] = '123 Lazy Lane'
        view.request.form['field.address2'] = ''
        view.request.form['field.city'] = ''
        view.request.form['field.state'] = ''
        view.request.form['field.zipcode'] = ''
        view.request.form['field.country'] = ''
        view.request.form['field.email'] = ''
        view.request.form['field.phone'] = ''
        view.request.form['field.fax'] = ''
        view.request.form['field.project'] = 'Slackin'
        view.request.form['field.objective'] = ''
        view.request.form['UPDATE_SUBMIT'] = True
        view()
        res = factory.select()
        self.failIf(res.count() != 1)
        self.failIf(shipment.contact_info_id != res[0].id)
        self.failIf(res[0].firstname != 'Bob')

    def testShippingInfo(self):
        project = self.project
        request = self.request
        shipment = self._addShipment(project, name='test_shipment')
        factory = api.factory('sip.ShippingInfo', context=project)
        res = factory.select()
        self.failIf(res.count() != 0)
        view = zapi.getView(shipment, 'shipping.html', request)
        self.failUnless(view())
        view.request.form['field.hand_delivered'] = ''
        view.request.form['field.carrier_id'] = ''
        view.request.form['field.account_number'] = ''
        view.request.form['field.tracking_number'] = 'X123K634574567'
        view.request.form['field.instructions'] = ''
        view.request.form['field.temp_method_id'] = ''
        view.request.form['field.newcontainer'] = ''
        view.request.form['field.housenumber'] = ''
        view.request.form['field.technician_id'] = ''
        view.request.form['UPDATE_SUBMIT'] = True
        view()
        res = factory.select()
        self.failIf(res.count() != 1)
        self.failIf(shipment.shipping_info_id != res[0].id)
        self.failIf(res[0].tracking_number != 'X123K634574567')

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestShipment))
    return suite

if __name__ == '__main__':
    unittest.main()
