# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: adapter.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface, Attribute
from sip.interfaces.sample import ISampleSchema, ISample

class IPrincipalRolesHelper(Interface):

    def availablePrincipals():
        """Return the available principals which we can assign
        roles in a given context"""

    def availableRoles():
        """Return the available roles which we can assign in
        a given context"""

class ISampleWithAliquots(ISample):
    """interface for adapter that associates aliquots with sample"""

    aliquots = Attribute('aliquot container')

    def update(other):
        """takes in another SampleWithAliquots and updates the aliquots"""

class ISampleWithOriginAliquots(ISample):
    """Interface for adapter that associates origin aliquots to sample"""

    origin_aliquots = Attribute('origin aliquot container')


from zope.interface.interface import InterfaceClass

class FacadeInterfaceClass(InterfaceClass):

    def __init__(self, *args, **kw):
        super(FacadeInterfaceClass, self).__init__(*args, **kw)
        self._targets = []

    def addTarget(self, iface):
        self._targets.append(iface)

    def names(self, all=False):
        r = {}
        for iface in self._targets:
            for name in iface.names(all):
                r[name] = None
        for name in super(FacadeInterfaceClass, self).names(all):
            r[name] = None
        return r.keys()

    def namesAndDescriptions(self, all=False):
        r = {}
        for iface in self._targets:
            for name, value in iface.namesAndDescriptions(all):
                if name not in r:
                    r[name] = value
        n_d = super(FacadeInterfaceClass, self).namesAndDescriptions(all)
        for name, value in n_d:
            if name not in r:
                r[name] = value
        return r.items()

    def __getitem__(self, name):
        for iface in self._targets:
            if name in iface:
                return iface[name]
        return super(FacadeInterfaceClass, self).__getitem__(name)

    def __contains__(self, name):
        for iface in self._targets():
            if name in iface:
                return True
        return super(FacadeInterfaceClass, self).__contains__(name)

    def providedBy(self, ob):
        for iface in self._targets:
            if iface.providedBy(ob):
                return True
        return super(FacadeInterfaceClass, self).providedBy(ob)

    def providedByInstancesOf(self, cls):
        for iface in self._targets:
            if iface.providedByInstancesOf(cls):
                return True
        return super(FacadeInterfaceClass, self).providedByInstancesOf(cls)

    def isOrExtends(self, iface):
        for iface in self._targets:
            if iface.isOrExtends(iface):
                return True
        return super(FacadeInterfaceClass, self).isOrExtends(iface)

FacadeInterface = FacadeInterfaceClass("FacadeInterface")

class IAugmentedSample(ISampleSchema):
    """ Sample with AdditionalSampleData """

    def additional_field_names():
        """ equivalent to schema.names() """

class IZopeAliquot(Interface):
    """Provides access to objects that relate to an aliquot
    so that the returned object is security checked and
    wrapped in a context.

    We do this on a separate adapter to avoid putting
    Zope-specific code on non-Zope-specific classes.
    """

    volume_unit = Attribute('Volume Unit')
    concentration_unit = Attribute('Concentration Unit')
    weight_unit = Attribute('Weight Unit')
    sample = Attribute('Sample')
    project = Attribute('Project')
    container = Attribute('Container')

class IZopeSample(Interface):
    """Provides access to objects that relate to a sample
    so that the returned object is security checked and
    wrapped in a context.

    We do this on a separate adapter to avoid putting
    Zope-specific code on non-Zope-specific classes.
    """

    project = Attribute('Project')
    type = Attribute('Sample Type')
