# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: aliquot.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface, Attribute
from zope.schema import TextLine, Text, Datetime, Int
from zope.schema import Float, Bool, Choice

from sip.i18n import _
from sip.interfaces import ISIPSchema, Unique, FieldFactory
from sip.descriptors import BoundCallableDescriptor
from sip.fields import BRField

def ispositive(value):
    if value:
        return value > 0
    return 1

class IAliquotSchema(ISIPSchema):
    """ """

    id = Int(title=_('Label'),
             required=True,
             readonly=True)

    label = Attribute('label')

    project_id = Attribute('project_id')

    sample_id = Int(title=_('Sample'),
                    required=True)

#    br1 = BRField(required=False)

    volume_id = Choice(
        title=_('Volume Unit'),
        description=_("The volume's unit of measure"),
        required=False,
        vocabulary='sip.vocab.available_volume_units')

    volume = Float(title=_('Volume'),
                   description=_("The aliquot's volume"),
                   constraint=ispositive,
                   required=False)

    weight_id = Choice(
        title=_('Weight Unit'),
        description=_("The weight's unit of measure"),
        required=False,
        vocabulary='sip.vocab.available_weight_units')

    weight = Float(title=_('Weight'),
                   description=_("Enter the aliquot's weight"),
                   constraint=ispositive,
                   required=False)

    concentration_id = Choice(
        title=_('Concentration Unit'),
        description=_("The concentration's unit of measure"),
        required=False,
        vocabulary='sip.vocab.available_concentration_units')

    concentration = Float(title=_('Concentration'),
                   description=_("The aliquot's concentration"),
                   constraint=ispositive,
                   required=False)

    container_id = Int(title=_('Container ID'),
                       description=_('Click to choose a container'),
                       required=False,
                       )

    depleted = Bool(title=_('Depleted'),
                    description=_('Is there any more sample left in this aliquot'),
                    required=False)

    is_low = Bool(title=_('Running Low'),
                  description=_('Aliquot is Running Low'),
                  required=False)

    aliquot_count = Int(title=_('Aliquot Count'),
                        description=_('Number of Chiclets left'),
                        required=False)

    comment = Text(title=_('Comment'),
                   description=_('A brief comment'),
                   required=False)


class IAliquotFuncs(Interface):
    """ """

    def getVolumeUnit():
        """ """

    def getVolumeString():
        """returns the volume as a string with the label from the unit"""

    def getWeightUnit():
        """ """

    def getWeightString():
        """returns the weight as a string with the label from the unit"""

    def getConcentrationUnit():
        """ """

    def getConcentrationString():
        """returns the concentration as a string with the label from the unit"""

    def getLocation():
        """ """

    def nextID(context):
        '''returns the next available id'''

    def getContainerType():
        ''' returns container type label or None if not in a container '''

class IAliquot(IAliquotSchema, IAliquotFuncs):
    """ """

    name = Attribute('Name')

    date = Datetime(title=_('Date'),
                    required=False)


class IAliquotContainer(ISIPSchema):
    """ """

    aliquot_id = Attribute('aliquot_id')
    container_id = Attribute('container_id')
