# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: project.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface, Attribute
from zope.schema import TextLine, Bool, Choice

from sip.i18n import _
from sip.interfaces import ISIPSchema
from sip.fields import FieldFactory, InAndOutField
from sip.utils import Unique
from sip.descriptors import BoundCallableDescriptor

class IProjectName(ISIPSchema):
    """ Project Name """

    name = FieldFactory(
        TextLine,
        title=_('Project Name'),
        description=_('A string to easily identify this project'),
        constraint=BoundCallableDescriptor(
        Unique(className='sip.Project', columnName='name')))

    require_commit_message = Bool(
        title=_('Require Edit Comment'),
        description=_(('Make the edit comment message required on '
                       'every change')),
        required=True,
        default=False)

class IProjectValidLocations(Interface):
    """ Locations for a project """

    valid_locations = InAndOutField(
        value_type=Choice(vocabulary='sip.vocab.locations'),
        title=_('Valid Locations'),
        description=_(('Locations which are valid in the context '
                       'of this project'))
        )

class IProjectValidTechnicians(Interface):
    """ Technicians for a project """

    valid_technicians = InAndOutField(
        value_type=Choice(vocabulary='sip.vocab.technicians'),
        title=_('Valid Technicians'),
        description=_(('Technicians that are valid in the context '
                       'of this project'))
        )

class IProjectValidSampleTypes(Interface):
    """ Sample Types for a Project """

    valid_sample_types = InAndOutField(
        value_type=Choice(vocabulary='sip.vocab.sample_types'),
        title=_('Valid Sample Types'),
        description=_('You can select one or more Sample Types to '
                      'be assigned to this Project by holding the '
                      'CONTROL key on your keyboard and select '
                      'the Sample Types'))

class IProjectAvailableSampleTypes(Interface):
    """ Sample Types Available for a Project """

    sample_type = Choice(
        title=_('Project Sample Types'),
        description=_(('Sample Types that are valid in the context '
                       'of this project')),
        vocabulary='sip.vocab.project_sample_types')

class IProjectSchema(ISIPSchema, IProjectName,
                     IProjectValidLocations,
                     IProjectValidTechnicians,
                     IProjectValidSampleTypes):
    """ Full project schema"""

class IProject(Interface):
    """Just a marker interface"""
