# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: units.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.schema import Bool, Choice
from zope.schema import TextLine, Text, Datetime, Int, Float
from zope.interface import Interface, Attribute

from sip.i18n import _
from sip.interfaces import ISIPSchema, Unique, FieldFactory
from sip.descriptors import BoundCallableDescriptor


class IUnit(ISIPSchema):

    name = Attribute('Name')

    exponent = Float(title=_('Exponent'),
                   required=True)

class IConcentrationUnit(IUnit):
    """ """

    label = FieldFactory(
        TextLine,
        title=_('Label'),
        required=True,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.ConcentrationUnit', columnName='label'))
        )

class IWeightUnit(IUnit):
    """ """

    label = FieldFactory(
        TextLine,
        title=_('Label'),
        required=True,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.WeightUnit', columnName='label'))
        )

class IVolumeUnit(IUnit):
    """ """
    label = FieldFactory(
        TextLine,
        title=_('Label'),
        required=True,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.VolumeUnit', columnName='label'))
        )

class IAppConcentrationUnits(Interface):
    """Available locations on the App"""

    available_concentrationunits = Choice(
        title=_('Available Concentration Units'),
        description=_(('Concentration Units that '
                       'are available for use')),
        required=True,
        vocabulary='sip.vocab.available_concentration_units')

class IAppWeightUnits(Interface):
    """Available locations on the App"""

    available_weightunits = Choice(
        title=_('Available Weight Units'),
        description=_('Weight Units that are available for use'),
        required=True,
        vocabulary='sip.vocab.available_weight_units')

class IAppVolumeUnits(Interface):
    """Available locations on the App"""

    available_volumeunits = Choice(
        title=_('Available Volume Units'),
        description=_('Volume Units that are available for use'),
        required=True,
        vocabulary='sip.vocab.available_volume_units')
