# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: query.py 749 2005-06-28 22:21:49Z sidnei $
"""

__metaclass__ = type

import sys
from zope.app import zapi
from zope.app.rdb import queryForResults
from zope.app.container.interfaces import IReadContainer
from sip.interfaces.adapter import ISampleWithAliquots
from sip.browser import api

class SearchType:
    aliquot = 'Aliquot'
    subjectid = 'SubjectID'
    otherid = 'OtherID'
    samplename = 'SampleName'
    sampletype = 'SampleType'
    project = 'Project'
    addinfo = 'AddInfo'
    sampleid = 'SampleID'

class SortOn:
    samplename = 'SampleName'
    dategen = 'DateGenerated'
    sampleid = 'SampleID'


class Query:

    def __init__(self, cmds, sorton = None):
        if len(cmds) > 1:
            cmds = [c for c in cmds if c.queryName != SearchType.project]
        self.cmds = cmds
        self.sorton = sorton

class QueryCommand:

    def __init__(self, search_type, **kwargs):
        self.queryName = search_type
        self.kwargs = kwargs


    def execute(self, ctx):
        funcname = 'do%sQuery' % self.queryName
        mod = sys.modules[self.__module__]
        func = getattr(mod, funcname)
        samples = func(self, ctx)
        return samples

def intersect(s1, s2):
    d1 = {}.fromkeys(s1)
    res = []
    for k in s2:
        if k in d1:
            res.append(k)
    return res

def _getAdaptedSamples(ctx, query, projids=None, sids=None):
    aliquots = []
    sample_lists = []
    slice_count = slice_end = 1000
    slice_start = 0
    if projids is not None:
        projids = map(int, projids)
        if len(projids) == 1:
            query += " AND project_id = %d" % projids[0]
        else:
            query += " AND project_id IN %s" % str(tuple(projids))

    factory = api.factory('sip.Sample', context=ctx)
    samples = factory.select(clause=query)
    if not samples or samples.count() == 0:
        return ()
    sample_count = samples.count()
    sample_info = dict([(s.id, s.project_id) for s in samples])
    sample_ids = sample_info.keys()
    project_ids = dict.fromkeys(sample_info.values()).keys()
    if sids is not None:
        sample_ids = intersect(sample_ids, sids)
    sample_ids.sort()
    tmp = {}.fromkeys(sample_ids)
    root = api.getRoot(ctx)
    projects = root['projects']
    sample_containers = sc = {}
    for p in project_ids:
        sc[p] = IReadContainer(projects['sip.Project.%d' % p])['samples']
    samples = filter(lambda s: s.id in tmp, samples)
    contained = lambda s: api.contained(s, sc[s.project_id],
                                        'sip.Sample.%d' % s.id)
    samples = map(contained, samples)
    adapt = lambda s: ISampleWithAliquots(s)
    samples = map(adapt, samples)
    return samples


def _setupQuery(kwargs):
    projids = kwargs['project_ids']
    if len(projids) == 1:
        query = "project_id = %d" % projids[0]
    else:
        query = "project_id IN %s" % str(tuple(projids))

    type_ids = kwargs.get('type_ids')
    if type_ids:
        if len(type_ids) == 1:
            query += " AND type_id = %d" % type_ids[0]
        else:
            query += " AND type_id IN %s" % str(tuple(type_ids))

    subject_ids = kwargs.get('subject_ids')
    if subject_ids:
        id_type = kwargs['id_type']
        if len(subject_ids) == 1:
            query += " AND %s = '%s'" % (id_type, subject_ids[0])
        else:
            query += " AND %s IN %s" % (id_type, str(tuple(subject_ids)))

    collect_dates = kwargs.get('collect_dates', {})
    if collect_dates and collect_dates.get('start_date') \
                     and collect_dates.get('end_date'):
        #" AND collection_date >= '%s' AND collection_date <= '%s'"\
        query +=\
        " AND collection_date >= date('%s') AND collection_date <= date('%s')"\
        % (collect_dates['start_date'], collect_dates['end_date'])

    entered_dates = kwargs.get('entered_dates', {})
    if entered_dates and entered_dates.get('start_date') \
                     and entered_dates.get('end_date'):
        #" AND date_generated >= '%s' AND date_generated <= '%s'"\
        query +=\
        " AND date_generated >= date('%s') AND date_generated <= date('%s')"\
        % (entered_dates['start_date'], entered_dates['end_date'])
    return query

def doProjectQuery(self, ctx):
    kwargs = self.kwargs
    query = _setupQuery(kwargs)
    return _getAdaptedSamples(ctx, query)

def _queryByAddInfo(ctx, field_id, op, value):
    factory = api.factory('sip.Sample', context=ctx)
    conn = factory._connection._connection
    cursor = conn.conn.cursor()
    query = "select * from select_on_info(%d, '%s', '%s') as id" % \
            (field_id, op, value)
    results = queryForResults(conn, query)
    sample_ids = [r.id for r in results]
    #cursor.callproc('select_on_info', [field_id, op, value])
    #results = cursor.fetchall()
    #sample_ids = [r[0] for r in results]
    return sample_ids

def doAddInfoQuery(self, ctx):
    kwargs = self.kwargs
    sids = []
    lists = []
    value_flag = False
    info_querys = kwargs.get('info_querys')
    if info_querys:
        for iq in info_querys:
            value = iq['value'].strip()
            if value:
                value_flag = True
                field_id = iq['field_id']
                op = iq['op']
                lists.append(_queryByAddInfo(ctx, field_id, op, value))
        lists = [l for l in lists if len(l)]
    if not value_flag:
        sids = None
    elif not lists:
        return ()
    if lists:
        if len(lists) == 1:
            sids = lists[0]
        else:
            sids = lists[0]
            for l in lists[1:]:
                sids = intersect(l,sids)
    query = _setupQuery(kwargs)
    results = _getAdaptedSamples(ctx, query, sids=sids)
    return results

def doSampleNameQuery(self, ctx):
    names = self.kwargs['names']
    if len(names) == 1:
        query = "name = '%s'" % names[0]
    else:
        query = "name IN %s" % str(tuple(names))
    projids = self.kwargs['project_ids']
    return _getAdaptedSamples(ctx, query, projids)

def doSampleIDQuery(self, ctx):
    ids = self.kwargs['sample_ids']
    if len(ids) == 1:
        query = "sample_id = %s" % ids[0]
    else:
        query = "sample_id IN %s" % str(tuple(ids))
    projids = self.kwargs['project_ids']
    return _getAdaptedSamples(ctx, query, projids)


def doOtherIDQuery(self, ctx):
    otherids = self.kwargs['ids']
    if len(otherids) == 1:
        query = "other_id = '%s'" % otherids[0]
    else:
        query = "other_id IN %s" % str(tuple(otherids))
    projids = self.kwargs['project_ids']
    return _getAdaptedSamples(ctx, query, projids)


def doAliquotQuery(self, ctx):
    factory = api.factory('sip.Aliquot', context=ctx)
    labels = self.kwargs['labels']
    if len(labels) == 1:
        query = "aliquot_id = %s" % labels[0]
    else:
        query = 'aliquot_id IN %s' % str(tuple(labels))
    aliquots = factory.select(clause = query)
    if aliquots.count() == 0:
        return ()
    sids = [a.sample_id for a in aliquots]

    factory = api.factory('sip.Sample', context=ctx)
    if len(sids) == 1:
        query = 'sample_id = %s' % sids[0]
    else:
        query = 'sample_id IN %s' % str(tuple(sids))
    projids = self.kwargs['project_ids']
    if len(projids) == 1:
        query += " AND project_id = %s" % projids[0]
    else:
        query += ' AND project_id IN %s' % str(tuple(projids))

    samples = factory.select(clause=query)
    sids = {}
    for s in samples:
        sids[s.id] = True
    root = api.getRoot(ctx)
    project = root['projects']['sip.Project.%d' % projids[0]]
    project = IReadContainer(project)
    alsource = project['aliquots']

    samples = [
        ISampleWithAliquots(api.contained(a, alsource, 'sip.Aliquot.%d' % a.id))
        for a in aliquots if a.sample_id in sids]
    return samples
