# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: sample.py 749 2005-06-28 22:21:49Z sidnei $
"""
__metaclass__ = type

from datetime import date
from sqlobject import *

from zope.interface import implements

from sqlos import SQLOS

from sip.browser import api
from sip.interfaces.sample import ISample

class Sample(SQLOS):

    implements(ISample)

    _idName='sample_id'
    _columns = [
        StringCol('name', length=255, notNull=0,
                  validator=validators.String(strip_spaces=1)),
        StringCol('other_id', length=255, notNull=0,
                  validator=validators.String(strip_spaces=1)),
        DateCol('date_generated', notNull=1, default=date.today),
        StringCol('comments', notNull=0,
                  validator=validators.String(strip_spaces=1)),
        StringCol('subject_id', length=255, default='',
                  validator=validators.String(strip_spaces=1)),
        ForeignKey('Project', name='project_id', cascade=False),
        ForeignKey('SampleType', name='type_id', cascade=False),
        ForeignKey('Technician', name='technician_id', cascade=False),
        ForeignKey('SubjectType', name='subject_type_id', cascade=False),
        BoolCol('derived', notNull=1, default=False),
        StringCol('commit_message', notNull=1),
        DateCol('collection_date', notNull=0),
        ]

    def _set_name(self, value):
        if hasattr(self, 'id'):
            if not repr(value) == repr(self.id):
                self._SO_set_name(value)
        else:
            # `id` may not exist yet. This
            # occurs during instantiation.
            self._SO_set_name(value)

    def _get_name(self):
        name = self._SO_get_name()
        if not name:
            return str(self.id)
        return name


import sys
class AdditionalSampleData(SQLOS):

    _idName = 'additional_sample_data_id'

    _columns = [
        ForeignKey('Sample', name='sample_id', cascade=False),
        ForeignKey('SchemaField', name='schema_field_id', cascade=False),
        StringCol('value', length=50, notNull=0, default=None),
        ]

    def getValue(self):
        klassname=self.schema_field_.type
        klass=klassname.split('.')[-1]
        module='.'.join(klassname.split('.')[:-1])
        klass_module=sys.modules[module]
        klass=getattr(klass_module, klass)
        value=self._SO_get_value()
        field_type, value = api.convertFieldAndValue(klass, value)
        return value

    def setValue(self, value):
        self._SO_set_value(value)

    value = property(getValue, setValue)
