# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: nullpolicy.py 749 2005-06-28 22:21:49Z sidnei $
"""
__metaclass__ = type

from zope.interface import implements
from zope.security.interfaces import ISecurityPolicy

class NullSecurityPolicy:

    implements(ISecurityPolicy)

    def __init__(self, ownerous=True, authenticated=True):
        """
            Two optional keyword arguments may be provided:

            ownerous -- Untrusted users can create code
                (e.g. Python scripts or templates),
                so check that code owners can access resources.
                The argument must have a truth value.
                The default is true.

            authenticated -- Allow access to resources based on the
                privileges of the authenticated user.
                The argument must have a truth value.
                The default is true.

                This (somewhat experimental) option can be set
                to false on sites that allow only public
                (unauthenticated) access. An anticipated
                scenario is a ZEO configuration in which some
                clients allow only public access and other
                clients allow full management.
        """

        self._ownerous = ownerous
        self._authenticated = authenticated

    checkPermission = lambda w, x, y, z: True

nullSecurityPolicy = NullSecurityPolicy()

from zope.security import checker

class Checker(checker.Checker):

    def __init__(self, permission_func,
                 setattr_permission_func=lambda name: None
                 ):
        pass

    def check_getattr(self, object, name):
        'See IChecker'
        return

    def check_setattr(self, object, name):
        'See IChecker'
        return

    def check(self, object, name):
        return

    def proxy(self, value):
        'See IChecker'
        return value

ProxyFactory = lambda x, y=None: x
selectChecker = lambda x: None

import sys
sys.modules['zope.security.checker.Checker'] = Checker
sys.modules['zope.security.checker.ProxyFactory'] = ProxyFactory
sys.modules['zope.security.checker.selectChecker'] = selectChecker
sys.modules['zope.security.proxy.ProxyFactory'] = ProxyFactory

checker.Checker = Checker
checker.ProxyFactory = ProxyFactory
checker.selectChecker = selectChecker

from zope.security import proxy
proxy.ProxyFactory = ProxyFactory

from zope.app.pagetemplate import engine
engine.ProxyFactory = ProxyFactory

from zope.app.publication import zopepublication
from zope.app.publication import publicationtraverse
from zope.app.publication import browser
zopepublication.ProxyFactory = ProxyFactory
publicationtraverse.ProxyFactory = ProxyFactory
browser.ProxyFactory = ProxyFactory
