# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: dashboard.py 758 2005-06-28 23:02:39Z sidnei $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.publisher.browser import BrowserView
from zope.app.container.interfaces import IReadContainer
from sip.browser import api
from zope.security.proxy import removeSecurityProxy

class Dashboard(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        project = api.getProject(context)
        self.can_add = api.checkSIPPermission(project, 'sip.AddShipment')
        self.can_edit = api.checkSIPPermission(project, 'sip.EditShipment')

    def info(self):
        info = {}
        direction = IReadContainer(self.context)
        funding_factory = api.factory('sip.FundingInfo',
                context = self.context)
        for key, direction in direction.items():
            sub = []
            d_adapter = IReadContainer(direction)
            # To insure states are shown in the right order. May only
            # work with outgoing direction but thats the only one now
            # anyway.
            for k in ['incomplete', 'intransit', 'received']:
                state = d_adapter[k]
                state_adapted = IReadContainer(state)
                shipments = {}
                for sid, shipment in state_adapted.items():
                    naked = removeSecurityProxy(shipment)
                    contact = bool(naked.contact_info_id)
                    shipping = bool(naked.shipping_info_id)
                    funding = funding_factory.selectBy(
                            shipment_id=shipment.id)
                    funding = bool(funding.count())
                    shipments[sid] = {'name':shipment.name,
                                      'contact_complete': contact,
                                      'shipping_complete': shipping,
                                      'funding_complete': funding,
                                      'item':shipment}
                sub.append({'name':state.name,
                           'item':state,
                           'showadd': k == 'incomplete',
                           'count':len(state_adapted),
                           'items':shipments.values()})
            info[key] = {'name':direction.name, 'item':direction,
                         'data':sub}
        return info.values()
