/*==============================================================*/
/* DBMS name:      Sybase AS Enterprise 12.5.1                  */
/* Created on:     5/27/2004 3:43:20 PM                         */
/*  Version 1.0.1 srr				*/
/*==============================================================*/


if exists (select 1
          from sysobjects
          where id = object_id('dbo.select_on_info')
          and type = 'P')
   drop procedure dbo.select_on_info
go


if exists (select 1
          from sysobjects
          where id = object_id('dbo.p_threshold_dump_tran')
          and type = 'P')
   drop procedure dbo.p_threshold_dump_tran
go


if exists (select 1
          from sysobjects
          where id = object_id('dbo.p_get_key_range')
          and type = 'P')
   drop procedure dbo.p_get_key_range
go


if exists (select 1
          from sysobjects
          where id = object_id('dbo.p_get_key_output')
          and type = 'P')
   drop procedure dbo.p_get_key_output
go


if exists (select 1
          from sysobjects
          where id = object_id('dbo.p_get_key')
          and type = 'P')
   drop procedure dbo.p_get_key
go


if exists (select 1
          from sysobjects
          where id = object_id('dbo.p_get_aliquot_ids')
          and type = 'P')
   drop procedure dbo.p_get_aliquot_ids
go


if exists (select 1
          from sysobjects
          where id = object_id('dbo.get_field_id')
          and type = 'P')
   drop procedure dbo.get_field_id
go


if exists (select 1
            from  sysobjects
            where id = object_id('ti_aliquot')
            and   type = 'TR')
   drop trigger ti_aliquot
go



create procedure dbo.get_field_id (
 @sampleid	integer,
 @fname		varchar(30),
 @stname		varchar(30)
 ) as
begin
	
	declare 	@field_id		integer, 
		@projid		integer,
		@stypeid		integer,
		@schemaid	integer
	
	select @projid = project_id, @stypeid = type_id
		from sample
		where sample_id = @sampleid
	
	if @stname != "" 
	begin
		select @stypeid = sample_type_id
		from sample_type
		where name = @stname
			and sample_type_id = @sampleid
		
	end	
	
	select @schemaid = sample_type_schema_id
		from sample_type_schema
		where project_id = @projid
	
	select @field_id = schema_field_id
		from schema_field
		where name = @fname
	
	return @field_id
		
end
go



create procedure dbo.p_get_aliquot_ids (
      @aint_needed integer,
      @aint_hold integer = 1440
                    ) as
BEGIN
/********************************************************************
Procedure:	p_get_aliquot_ids

Arguments:
	@aint_needed	integer	
				Number of aliquot ids needed
	@aint_hold	integer
				Age in minutes before 'reserve' id
				can be used or set to 'free'
				default value is 1440 (1 day)
				
				
				
Returns: If successful returns number of ids and result set
	containing sequential number and aliquot id.
	 
	 If fails, returns negative number.
	-101	Non-Positive Argument
	-112	NULL Argument
	-103	Bad values returned from p_get_key_range
	-104	NULL value returned from p_get_key_range

Scott R. Rouse		02-10-04	Created

********************************************************************/

declare @int_free	integer,	@errno		integer
declare @key_start	numeric(18,0),	@key_end	numeric(18,0)
declare @int_rc		integer, 	@current_key	integer,	@looper	integer


-- check for positive, non-null args
if @aint_needed <=0 or @aint_hold <=0
	return -101
if @aint_needed is null
	return -112


-- create a temp table with an identity column
create table #free (
	counter                         numeric(10,0)                    identity  ,
	id                              numeric(18,0)                    not null  
	)

	create unique clustered index pk_tmpfree_counter on #free(counter)
	create unique nonclustered index pk_tmpfree_id on #free(id)

--begin transaction
begin tran id_tran

lock table aliquot_allocation in exclusive mode
	wait 20

-- Get number of currently 'free' ids
select @int_free = count(aliquot_id)
	from aliquot_allocation holdlock
	where status = 'free'
select @errno = @@error


if @aint_needed > @int_free and (@errno != 0)
begin
	-- don't have enough free
	-- purge expired reserves

	update aliquot_allocation
	set status = 'free'
	where status = 'reserved'
		and datediff(minute,tstamp,getdate()) > @aint_hold

	-- update free count
	select @int_free = count(aliquot_id)
	from aliquot_allocation holdlock
	where status = 'free'

	
	-- if end
end

if @aint_needed > @int_free
 begin
-- Don't have enough free ids.  just pull new numbers.
-- function gets start and stop range for IDs
exec p_get_key_range 'aliquot_id', @aint_needed, @key_start = @key_start output, @key_end = @key_end output
	-- need to add arg here to catch return value and trap if an error occurs.
	if (@key_start is null) or (@key_end is null)
	begin
	
			select @errno = -104
			goto error
	
	end

	if @key_end >= @key_start 
	begin
		-- Loop through and add a row and id from each requested aliquot_id
		--select "in loop"
		-- temp table #free is local to this proc. not currently checking returns
		select @looper = 1
		select @int_rc = 0, @current_key = @key_start
		while @looper <= @aint_needed
		begin
			insert into #free(id)
			values(@current_key)
			select @looper = @looper + 1, @current_key = @current_key + 1
		end -- while		
	
	end
	else
	begin
		-- bad values returned from get_key_range
		select @errno = -103
		goto error
	end		
	
-- if aint_needed end
 end
else
 begin
	-- have enough free ids
	-- use them
	-- insert into temp table all free rows, which now have a sequential counter row
	insert into #free(id)
		select aliquot_id from aliquot_allocation
		where status = 'free'
		order by aliquot_id
	

	-- delete from temp table #free ids we don't need
	delete from #free where counter > @aint_needed

-- should rows be deleted here?	
	delete aliquot_allocation
		from #free pf
		where pf.id = aliquot_allocation.aliquot_id

	-- update status to reflect that these ids are now reserved?
--	update aliquot_allocation
--		set status = 'reserved', tstamp = getdate()
--		from #free pf
--			where pf.id = aliquot_allocation.aliquot_id

	
 -- else end		
 end

-- Return Result set
commit tran id_tran
select @int_rc = count(*) from #free

-- to test only
--insert into xproof
--select #free.counter, #free.id from #free
-- end test

select #free.counter, #free.id from #free
--select #free.id from #free

	
	return @int_rc


error:
begin
	rollback tran id_tran
	
	return	@errno	 

end

-- proc end
END
go



create procedure dbo.p_get_key (
     @counter_type  char(60)
                                        ) as
Begin
/**********************************************************
*Procedure Name: p_get_key
*Database: animal
*Server: vroomfondel
*SQL-Builder4.2
*
*Business Function : Increments given counter and returns current value.

*Author BF: Scott R. Rouse      Date BF: 06/11/2003
*
*Author: sro          Date: 06/11/2003
*  Comment: creation
*	Adapted from p_get_next_counter
*
*
*	03-02-04		add exclusive lock after begin_tran
			proc returns counter
*********************************************************/
declare @cnt integer, @counter_out integer, @myerror integer

select @cnt = -1

begin tran tran_counter

	lock table sip_counters in exclusive mode 	wait 5
	
	select @myerror = @@error
-- will return @@error of 12207 is table can not be locked.
	if @myerror  = 12207 
	begin
          		rollback tran tran_counter
          		select @counter_out = -99
            		return @counter_out
     	end


    select @cnt = counter from sip_counters holdlock
            where counter_type = @counter_type
     
     
    if @cnt = -1 
     begin
            select @counter_out = -1
            rollback tran tran_counter
   end
    else
     begin
            
            -- Increments counter                     
             update sip_counters
             set counter = @cnt +1
                 where @counter_type = counter_type
                 
-- Pulls new count
             select @counter_out = counter from sip_counters
                                where @counter_type = counter_type
             
             -- If it incremented commit                   
             if @cnt + 1 = @counter_out                   
                begin
                     commit tran tran_counter
                end
             else
                begin
                    select @counter_out = -1
                    rollback tran tran_counter
                end
                           
     end
-- returns counter
return @counter_out
end
go



create procedure dbo.p_get_key_output @counter_type  char(60), @counter_out int output as
BEGIN
/**********************************************************
*Procedure Name: p_get_key
*Database: animal
*Server: vroomfondel
*SQL-Builder4.2
*
*Business Function : Increments given counter and returns current value.

*Author BF: Scott R. Rouse      Date BF: 06/11/2003
*
*Author: sro          Date: 06/11/2003
*  Comment: creation
*	Adapted from p_get_next_counter
*
*
*	03-02-04		add exclusive lock after begin_tran
			proc returns counter
*********************************************************/
declare @cnt integer,  @myerror integer

select @cnt = -1

begin tran tran_counter

	lock table sip_counters in exclusive mode 	wait 5
	
	select @myerror = @@error
--will return @@error of 12207 is table can not be locked.
	if @myerror  = 12207 
	begin
          		rollback tran tran_counter
          		select @counter_out = -99
            		return @counter_out
     	end


    select @cnt = counter from sip_counters holdlock
            where counter_type = @counter_type
     
     
    if @cnt = -1 
     begin
            select @counter_out = -1
            rollback tran tran_counter
     end
    else
     begin
            
            -- Increments counter             
             update sip_counters
             set counter = @cnt +1
                 where @counter_type = counter_type
                 
             -- Pulls new count
             select @counter_out = counter from sip_counters
                      where @counter_type = counter_type
             
             -- If it incremented commit                   
             if @cnt + 1 = @counter_out                   
                begin
                     commit tran tran_counter
                end
             else
                begin
                    select @counter_out = -1
                    rollback tran tran_counter
                end
                           
     end
-- returns counter
return @counter_out		
	END
go



create procedure dbo.p_get_key_range (
      @counter_type varchar(40),
      @keys_needed	integer,
      @key_start		numeric(18,0) = null output, 
      @key_end		numeric(18,0) =null output 
 
                    ) as
BEGIN
/**********************************************************
Procedure:	p_get_key range

Arguments:
	@counter_type	varchar(40)	
				Name of counter
	@keys_needed	integer
				Number of keys reqested
	@key_start	numeric(18,0)  output
				Output variable that holds
				the 1st valid key.
	@key_stop	numeric(18,0)  output
				Output variable that holds
				the last valid key.
				
				
				
Returns: If successful returns output variables holding
	 requested key info.
	Procedure returns 0
	
	 If fails, returns negative number.
	-101	Non-Positive Argument
	-X	Error returned from the database
		

SRR	02-06-04

*********************************************************/

declare @rowcnt 	integer,
	@errno 		integer,
	@counter	numeric(18,0)
	

if @keys_needed <= 0 
	return -101



-- starts transaction	
begin tran counter_trans 

lock table sip_counters in exclusive mode
	wait 20

-- gets current_value	
select @counter = counter
	  from sip_counters holdlock	
	 where counter_type = @counter_type
					
select @errno = @@error

if @errno != 0
 begin                    
	goto error
 end


if @counter >= 0
 begin
 	-- sets output values
 	select @key_start = @counter, @key_end = @counter + @keys_needed - 1
	-- update counter
    	update sip_counters
    	set counter = @key_start + @keys_needed
    		where counter_type = @counter_type
	
	select @errno = @@error
	if @errno != 0	
	 begin                    
	  goto error
	 end

	
     
     
 end
	
 	
	
commit tran return0 

	
error:
begin
        
	rollback tran return @errno
	

end

END
go



create procedure dbo.p_threshold_dump_tran as
BEGIN
		dump transaction sip with truncate_only

        print "LOG DUMP threshold:"
	END
go



create procedure dbo.select_on_info (
 @field_id			integer,
 @op			varchar(255),
 @myvalue		varchar(255)
 ) as
begin
	-- Constants
	declare	@int_type	varchar(50),
		@float_type	varchar(50),
		@date_type	varchar(50),
		@bool_type	varchar(50),
		@text_type	varchar(50),
		@textline_type	varchar(50)	 
	
	
	select	@int_type	= 	'zope.schema._bootstrapfields.Int',
		@float_type	= 	'zope.schema._field.Float',
		@date_type	= 	'zope.schema._field.Datetime',
		@bool_type	= 	'zope.schema._bootstrapfields.Bool',
		@text_type	= 	'zope.schema._bootstrapfields.Text',  
		@textline_type	= 	'zope.schema._bootstrapfields.TextLine'

	-- Local Vars
	declare	@data_type	varchar(50),
		@convert_type	varchar(50),
		@query		varchar,
		@query_rec	integer
		
	select @data_type = type 
		from schema_field
		where schema_field_id = @field_id
	
	select convert_type = 		
	case 
	when @data_type = @int_type	then  'INTEGER'
	when @data_type = @float_type	then  'NUMERIC'
	when @data_type = @date_type	then  'DATE'
	when @data_type = @bool_type	then  'INTEGER'
	else 'VARCHAR'
	end
		
		
		
	
		



		
end
go



  Begin
/**********************************************************
*Trigger Name: ti_aliquot
*Database: sip
*Server: lunkwill
*SQL-Builder4.3
*
*Business Function : Deletes from aloquot_allocation when the id is inserted into aloquots.

*Author BF: SRR     Date BF: 05/24/2004
*
*********************************************************/

  delete aliquot_allocation
    from aliquot_allocation, inserted
    where aliquot_allocation.aliquot_id = inserted.aliquot_id


end

