##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: __init__.py 5839 2004-07-31 16:24:13Z dreamcatcher $
"""

from persistent import Persistent

from zope.interface import implements
from zope.component import IFactory
from zope.app import zapi
from zope.app.event import publish
from zope.app.event.objectevent import ObjectCreatedEvent
from zope.app.container.interfaces import IContainer
from zope.app.container.interfaces import IContained
from zope.app.container.contained import ContainedProxy
from zope.app.container.contained import Contained
from zope.app.container.browser.adding import Adding
from zope.app.container.constraints import checkFactory
from zope.app.location.interfaces import ILocation
from zope.proxy import removeAllProxies
from zope.proxy import sameProxiedObjects

from sqlos.interfaces import ISQLObject
from sqlos.interfaces.container import ISQLObjectContainer
from sqlos.interfaces.container import ISQLObjectAdding
from sqlos import getFactory

from sqlobject import *
from sqlobject.main import CreateNewSQLObject

def contained(obj, parent=None, name=None):
    """An implementation of zope.app.container.contained.contained
    that doesn't generate events, for internal use.
    """
    if (parent is None):
        raise TypeError, 'Must provide a parent'

    if not IContained.providedBy(obj):
        if ILocation.providedBy(obj):
            directlyProvides(obj, IContained, directlyProvidedBy(obj))
        else:
            obj = ContainedProxy(obj)

    oldparent = obj.__parent__
    oldname = obj.__name__

    if (oldparent is None) or not (oldparent is parent
                                   or sameProxiedObjects(oldparent, parent)):
        obj.__parent__ = parent

    if oldname != name and name is not None:
        obj.__name__ = name

    return obj


class SQLObjectContainer(Persistent, Contained):

    implements(ISQLObjectContainer)

    def __init__(self):
        pass

    def allowedFactories(self):
        items = zapi.getFactoriesFor(ISQLObject, context=self)
        names = [name for name, factory in items]
        f = lambda name: (name, zapi.getUtility(IFactory, name))
        factories = map(f, names)
        a = lambda f, container=self: (checkFactory(container, None, f[1])
                                       and f[0] or None)
        allowed = filter(None, map(a, factories))
        return allowed

    def keys(self):
        """ Return a sequence-like object containing the names
        associated with the objects that appear in the folder
        """
        for name, obj in self.items(): yield name

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        """ Return a sequence-like object containing the objects that
        appear in the folder.
        """
        for name, obj in self.items(): yield obj

    def items(self):
        """Return a sequence-like object containing tuples of the form
        (name, object) for the objects that appear in the folder.
        """
        for factoryName in self.allowedFactories():
            factory = getFactory(factoryName)
            for obj in factory.select():
                name = '%s.%s' % (factoryName, obj.id)
                yield (name, contained(obj, parent=self, name=name))

    def __getitem__(self, name):
        """Return the named object, or the value of the default
           argument if given and the named object is not found.
           If no default is given and the object is not found a
           KeyError is raised.
        """
        try:
            parts = name.split('.')
            id = int(parts[-1])
            factoryName = '.'.join(parts[:-1])
        except ValueError:
            raise KeyError, name

        if factoryName not in self.allowedFactories():
            raise KeyError, name

        factory = getFactory(factoryName)
        try:

            obj = factory.get(id)
            return contained(obj, parent=self, name=name)
        except SQLObjectNotFound:
            raise KeyError, name

    def get(self, name, default=None):
        """Return the named object, or the value of the default
           argument if given and the named object is not found.
           If no default is given and the object is not found a
           KeyError is raised.
        """
        try:
            return self[name]
        except KeyError:
            return default

    def __contains__(self, name):
        """Return true if the named object appears in the folder."""
        return self.get(name, None) is not None

    def __len__(self):
        """Return the number of objects in the folder."""
        i = 0
        for k in self.keys(): i += 1
        return i

    def __delitem__(self, name):
        """Delete the named object from the container.

        Raises a KeyError if the object is not found.
        """
        obj = self[name]
        obj.destroySelf()

    def __setitem__(self, name, content):
        return name

class SQLObjectAdding(Adding):
    """Custom adding view for SQLObject objects."""

    implements(ISQLObjectAdding)

    def __init__(self, context, request):
        super(SQLObjectAdding, self).__init__(context, request)

    def add(self, content):
        container = IContainer(self.context)
        klass = content.__class__
        klass = removeAllProxies(klass)
        package = klass.__module__.split('.')[0]
        className = klass.__name__
        id = content.id

        name = '%s.%s.%s' % (package, className, id)
        container[name] = content
        return container[name]

    def action(self, type_name=''):
        if not type_name:
            raise UserError(_(u"You must select the type of object to add."))

        if type_name.startswith('@@'):
            type_name = type_name[2:]


        if zapi.queryView(self, type_name, self.request) is not None:
            url = "%s/%s" % (
                zapi.getView(self, "absolute_url", self.request),
                type_name)
            self.request.response.redirect(url)
            return

        content = zapi.createObject(self, type_name, CreateNewSQLObject)
        publish(self.context, ObjectCreatedEvent(content))

        self.add(content)
        self.request.response.redirect(self.nextURL())

    def namesAccepted(self):
        return not ISQLObjectContainer.providedBy(self.context)
