##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: __init__.py 5475 2004-07-07 15:52:34Z dreamcatcher $
"""

from sqlos.interfaces import ISQLObject
from sqlos.event.interfaces import expired
from zope.security.proxy import removeSecurityProxy
from sqlobject import SQLObjectNotFound

def sqlobjectModifiedSubscriber(event):
    obj = event.object
    if not ISQLObject.providedBy(obj): return

    # Send 'UPDATE' query, if not sent yet.
    # If we don't do this, the cache will be cleared
    # and if we try to re-fetch the object we will
    # lose the new values.
    obj.syncUpdate()

    expired(obj)

sqlobjectCreatedSubscriber = sqlobjectModifiedSubscriber

def sqlobjectExpiredSubscriber(event):
    obj = event.object

    if not ISQLObject.providedBy(obj): return

    from sqlos.connection import connCache
    for connection in connCache.values():
        connection.cache.expire(obj.id, obj.__class__)

    # Expire object values and re-sync from database. The transaction
    # has either been aborted or committed.
    obj_id = obj.id
    obj.expire()
    try:
        # Calling sync() would commit the changes because expire()
        # doesn't clear _SO_createValues. So we do more or less the
        # same as sync() does by calling _init() and setting _expired
        # to False.
        unproxied = removeSecurityProxy(obj)
        unproxied._init(obj_id)
        unproxied._expired = False
    except SQLObjectNotFound:
        # Object may have been deleted or un-created
        pass
